@@ -30,24 +30,22 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-from .log_utils import logger
-from . import utils
+import asyncio
+import contextlib
+from typing import Dict, Set
 
-subscriptions = {}
-SKIPLOG = set()
-qtile = None
+from libqtile import utils
+from libqtile.log_utils import logger
 
-
-def init(q):
-    global qtile
-    qtile = q
+subscriptions = {}  # type: Dict
+SKIPLOG = set()  # type: Set
 
 
 def clear():
     subscriptions.clear()
 
 
-class Subscribe(object):
+class Subscribe:
     def __init__(self):
         hooks = set([])
         for i in dir(self):
@@ -59,6 +57,7 @@ class Subscribe(object):
         lst = subscriptions.setdefault(event, [])
         if func not in lst:
             lst.append(func)
+        return func
 
     def startup_once(self, func):
         """Called when Qtile has started on first start
@@ -90,6 +89,24 @@ class Subscribe(object):
         """
         return self._subscribe("startup_complete", func)
 
+    def shutdown(self, func):
+        """Called before qtile is shutdown
+
+        **Arguments**
+
+        None
+        """
+        return self._subscribe("shutdown", func)
+
+    def restart(self, func):
+        """Called before qtile is restarted
+
+        **Arguments**
+
+        None
+        """
+        return self._subscribe("restart", func)
+
     def setgroup(self, func):
         """Called when group is changed
 
@@ -104,7 +121,6 @@ class Subscribe(object):
 
         **Arguments**
 
-            * qtile manager instance
             * name of new group
         """
         return self._subscribe("addgroup", func)
@@ -114,7 +130,6 @@ class Subscribe(object):
 
         **Arguments**
 
-            * qtile manager instance
             * name of deleted group
         """
         return self._subscribe("delgroup", func)
@@ -129,7 +144,8 @@ class Subscribe(object):
         return self._subscribe("changegroup", func)
 
     def focus_change(self, func):
-        """Called when focus is changed
+        """Called when focus is changed, including moving focus between groups or when
+        focus is lost completely
 
         **Arguments**
 
@@ -151,19 +167,11 @@ class Subscribe(object):
 
         **Arguments**
 
-        None
+            * ``Group`` receiving the new window
+            * ``Window`` added to the group
         """
         return self._subscribe("group_window_add", func)
 
-    def window_name_change(self, func):
-        """Called whenever a windows name changes
-
-        **Arguments**
-
-        None
-        """
-        return self._subscribe("window_name_change", func)
-
     def client_new(self, func):
         """Called before Qtile starts managing a new client
 
@@ -172,7 +180,7 @@ class Subscribe(object):
 
         **Arguments**
 
-            * ``window.Window`` object
+            * ``Window`` object
 
         Examples
         --------
@@ -184,7 +192,7 @@ class Subscribe(object):
                 if c.name == "xterm":
                     c.togroup("a")
                 elif c.name == "dzen":
-                    c.static(0)
+                    c.cmd_static(0)
         """
         return self._subscribe("client_new", func)
 
@@ -196,7 +204,7 @@ class Subscribe(object):
 
         **Arguments**
 
-            * ``window.Window`` object of the managed window
+            * ``Window`` object of the managed window
         """
         return self._subscribe("client_managed", func)
 
@@ -205,30 +213,16 @@ class Subscribe(object):
 
         **Arguments**
 
-            * ``window.Window`` object of the killed window.
+            * ``Window`` object of the killed window.
         """
         return self._subscribe("client_killed", func)
 
-    def client_state_changed(self, func):
-        """Called whenever client state changes
-
-        Never fires
-        """
-        return self._subscribe("client_state_changed", func)
-
-    def client_type_changed(self, func):
-        """Called whenever window type changes
-
-        Never fires
-        """
-        return self._subscribe("client_type_changed", func)
-
     def client_focus(self, func):
-        """Called whenever focus changes
+        """Called whenever focus moves to a client window
 
         **Arguments**
 
-            * ``window.Window`` object of the new focus.
+            * ``Window`` object of the new focus.
         """
         return self._subscribe("client_focus", func)
 
@@ -237,14 +231,16 @@ class Subscribe(object):
 
         **Arguments**
 
-            * ``window.Window`` of window entered
+            * ``Window`` of window entered
         """
         return self._subscribe("client_mouse_enter", func)
 
     def client_name_updated(self, func):
         """Called when the client name changes
 
-        Never fires
+        **Arguments**
+
+            * ``Window`` of client with updated name
         """
         return self._subscribe("client_name_updated", func)
 
@@ -253,7 +249,7 @@ class Subscribe(object):
 
         **Arguments**
 
-            * ``window.Window`` of client with hint change
+            * ``Window`` of client with hint change
         """
         return self._subscribe("client_urgent_hint_changed", func)
 
@@ -272,7 +268,7 @@ class Subscribe(object):
 
         **Arguments**
 
-            * ``window.Window`` of client with changed icon
+            * ``Window`` of client with changed icon
         """
         return self._subscribe("net_wm_icon_change", func)
 
@@ -299,27 +295,25 @@ class Subscribe(object):
         return self._subscribe("selection_change", func)
 
     def screen_change(self, func):
-        """Called when a screen is added or screen configuration is changed (via xrandr)
-
-        Common usage is simply to call ``qtile.cmd_restart()`` on each event
-        (to restart qtile when there is a new monitor):
+        """Called when the output configuration is changed (e.g. via randr in X11).
 
         **Arguments**
 
-            * qtile manager instance
-            * ``xproto.randr.ScreenChangeNotify`` event
+            * ``xproto.randr.ScreenChangeNotify`` event (X11) or None (Wayland).
 
-        Examples
-        --------
-
-        ::
-
-            @libqtile.hook.subscribe.screen_change
-            def restart_on_randr(qtile, ev):
-                qtile.cmd_restart()
         """
         return self._subscribe("screen_change", func)
 
+    def screens_reconfigured(self, func):
+        """Called once ``qtile.cmd_reconfigure_screens`` has completed (e.g. if
+        ``reconfigure_screens`` is set to ``True`` in your config).
+
+        **Arguments**
+
+        None
+        """
+        return self._subscribe("screens_reconfigured", func)
+
     def current_screen_change(self, func):
         """Called when the current screen (i.e. the screen with focus) changes
 
@@ -329,6 +323,25 @@ class Subscribe(object):
         """
         return self._subscribe("current_screen_change", func)
 
+    def enter_chord(self, func):
+        """Called when key chord begins
+
+        **Arguments**
+
+            * name of chord(mode)
+        """
+        return self._subscribe("enter_chord", func)
+
+    def leave_chord(self, func):
+        """Called when key chord ends
+
+        **Arguments**
+
+        None
+        """
+        return self._subscribe("leave_chord", func)
+
+
 subscribe = Subscribe()
 
 
@@ -337,26 +350,43 @@ class Unsubscribe(Subscribe):
     This class mirrors subscribe, except the _subscribe member has been
     overridden to removed calls from hooks.
     """
+
     def _subscribe(self, event, func):
         lst = subscriptions.setdefault(event, [])
         try:
             lst.remove(func)
         except ValueError:
             raise utils.QtileError(
-                "Tried to unsubscribe a hook that was not"
-                " currently subscribed"
+                "Tried to unsubscribe a hook that was not" " currently subscribed"
             )
 
+
 unsubscribe = Unsubscribe()
 
 
+def _fire_async_event(co):
+    loop = None
+    with contextlib.suppress(RuntimeError):
+        loop = asyncio.get_running_loop()
+
+    if loop is None:
+        asyncio.run(co)
+    else:
+        asyncio.ensure_future(co)
+
+
 def fire(event, *args, **kwargs):
     if event not in subscribe.hooks:
         raise utils.QtileError("Unknown event: %s" % event)
     if event not in SKIPLOG:
-        logger.info("Internal event: %s(%s, %s)", event, args, kwargs)
+        logger.debug("Internal event: %s(%s, %s)", event, args, kwargs)
     for i in subscriptions.get(event, []):
         try:
-            i(*args, **kwargs)
-        except:
+            if asyncio.iscoroutinefunction(i):
+                _fire_async_event(i(*args, **kwargs))
+            elif asyncio.iscoroutine(i):
+                _fire_async_event(i)
+            else:
+                i(*args, **kwargs)
+        except:  # noqa: E722
             logger.exception("Error in hook %s", event)
