@@ -29,13 +29,13 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-from .base import Layout
-from .. import drawer, hook, window
+from libqtile import hook
+from libqtile.layout.base import Layout
 
-to_superscript = dict(zip(map(ord, u'0123456789'), map(ord, u'⁰¹²³⁴⁵⁶⁷⁸⁹')))
+to_superscript = dict(zip(map(ord, u"0123456789"), map(ord, u"⁰¹²³⁴⁵⁶⁷⁸⁹")))
 
 
-class TreeNode(object):
+class TreeNode:
     def __init__(self):
         self.children = []
         self.parent = None
@@ -87,9 +87,7 @@ class TreeNode(object):
     def add_superscript(self, title):
         """Prepend superscript denoting the number of hidden children"""
         if not self.expanded and self.children:
-            return u"{:d}".format(
-                len(self.children)
-            ).translate(to_superscript).encode('utf-8') + title
+            return "{:d}".format(len(self.children)).translate(to_superscript) + title
         return title
 
     def get_first_window(self):
@@ -100,10 +98,11 @@ class TreeNode(object):
         """
         if isinstance(self, Window):
             return self
-        for i in self.children:
-            node = i.get_first_window()
-            if node:
-                return node
+        if self.expanded:
+            for i in self.children:
+                node = i.get_first_window()
+                if node:
+                    return node
 
     def get_last_window(self):
         """Find the last Window under this node
@@ -111,10 +110,11 @@ class TreeNode(object):
         Finds last `Window` by depth-first search, otherwise returns self if
         this is a `Window`.
         """
-        for i in reversed(self.children):
-            node = i.get_last_window()
-            if node:
-                return node
+        if self.expanded:
+            for i in reversed(self.children):
+                node = i.get_last_window()
+                if node:
+                    return node
         if isinstance(self, Window):
             return self
 
@@ -147,7 +147,7 @@ class TreeNode(object):
 
 class Root(TreeNode):
     def __init__(self, sections, default_section=None):
-        super(Root, self).__init__()
+        super().__init__()
         self.sections = {}
         for section in sections:
             self.add_section(section)
@@ -173,7 +173,7 @@ class Root(TreeNode):
             parent = hint.parent
 
         if parent is None:
-            sect = getattr(win, 'tree_section', None)
+            sect = getattr(win, "tree_section", None)
             if sect is not None:
                 parent = self.sections.get(sect)
 
@@ -203,7 +203,7 @@ class Root(TreeNode):
         sec = self.sections[name]
         # move the children of the deleted section to the previous section
         # if delecting the first section, add children to second section
-        idx = min(self.children.index(sec), 1)
+        idx = max(self.children.index(sec), 1)
         next_sec = self.children[idx - 1]
         # delete old section, reparent children to next section
         del self.children[idx]
@@ -211,43 +211,34 @@ class Root(TreeNode):
         for i in sec.children:
             i.parent = next_sec
 
+        del self.sections[name]
+
 
 class Section(TreeNode):
     def __init__(self, title):
-        super(Section, self).__init__()
+        super().__init__()
         self.title = title
 
     def draw(self, layout, top, level=0):
         del layout._layout.width  # no centering
         # draw a horizontal line above the section
-        layout._drawer.draw_hbar(
-            layout.section_fg,
-            0,
-            layout.panel_width,
-            top,
-            linewidth=1
-        )
+        layout._drawer.draw_hbar(layout.section_fg, 0, layout.panel_width, top, linewidth=1)
         # draw the section title
         layout._layout.font_size = layout.section_fontsize
         layout._layout.text = self.add_superscript(self.title)
         layout._layout.colour = layout.section_fg
-        layout._layout.draw(
-            x=layout.section_left,
-            y=top + layout.section_top
-        )
-        top += layout._layout.height + \
-            layout.section_top + \
-            layout.section_padding
+        layout._layout.draw(x=layout.section_left, y=top + layout.section_top)
+        top += layout._layout.height + layout.section_top + layout.section_padding
 
         # run the TreeNode draw to draw children (if expanded)
-        top = super(Section, self).draw(layout, top, level)
+        top = super().draw(layout, top, level)
 
         return top + layout.section_bottom
 
 
 class Window(TreeNode):
     def __init__(self, win):
-        super(Window, self).__init__()
+        super().__init__()
         self.window = win
         self._title_top = None
 
@@ -261,6 +252,9 @@ class Window(TreeNode):
         if self.window is layout._focused:
             fg = layout.active_fg
             bg = layout.active_bg
+        elif self.window.urgent:
+            fg = layout.urgent_fg
+            bg = layout.urgent_bg
         else:
             fg = layout.inactive_fg
             bg = layout.inactive_bg
@@ -268,10 +262,7 @@ class Window(TreeNode):
         layout._layout.width = layout.panel_width - left
         # get a text frame from the above
         framed = layout._layout.framed(
-            layout.border_width,
-            bg,
-            layout.padding_x,
-            layout.padding_y
+            layout.border_width, bg, layout.padding_x, layout.padding_y
         )
         # draw the text frame at the given point
         framed.draw_fill(left, top)
@@ -279,13 +270,13 @@ class Window(TreeNode):
         top += framed.height + layout.vspace + layout.border_width
 
         # run the TreeNode draw to draw children (if expanded)
-        return super(Window, self).draw(layout, top, level + 1)
+        return super().draw(layout, top, level + 1)
 
     def button_press(self, x, y):
         """Returns self if clicked on title else returns sibling"""
         if self._title_top <= y < self._children_top:
             return self
-        return super(Window, self).button_press(x, y)
+        return super().button_press(x, y)
 
     def remove(self):
         """Removes this Window
@@ -309,15 +300,55 @@ class TreeTab(Layout):
     """Tree Tab Layout
 
     This layout works just like Max but displays tree of the windows at the
-    left border of the screen, which allows you to overview all opened windows.
+    left border of the screen_rect, which allows you to overview all opened windows.
     It's designed to work with ``uzbl-browser`` but works with other windows
     too.
+
+    The panel at the left border contains sections, each of which contains
+    windows. Initially the panel looks like flat lists inside its
+    section, and looks like trees if some of the windows are "moved" left or
+    right.
+
+    For example, it looks like below with two sections initially:
+
+    ::
+
+        +------------+
+        |Section Foo |
+        +------------+
+        | Window A   |
+        +------------+
+        | Window B   |
+        +------------+
+        | Window C   |
+        +------------+
+        |Section Bar |
+        +------------+
+
+    And then it will look like below if "Window B" is moved right and "Window C"
+    is moved right too:
+
+    ::
+
+        +------------+
+        |Section Foo |
+        +------------+
+        | Window A   |
+        +------------+
+        |  Window B  |
+        +------------+
+        |   Window C |
+        +------------+
+        |Section Bar |
+        +------------+
     """
 
     defaults = [
         ("bg_color", "000000", "Background color of tabs"),
         ("active_bg", "000080", "Background color of active tab"),
         ("active_fg", "ffffff", "Foreground color of active tab"),
+        ("urgent_bg", "ff0000", "Background color of urgent tab"),
+        ("urgent_fg", "ffffff", "Foreground color of urgent tab"),
         ("inactive_bg", "606060", "Background color of inactive tab"),
         ("inactive_fg", "ffffff", "Foreground color of inactive tab"),
         ("margin_left", 6, "Left margin of tab panel"),
@@ -328,7 +359,7 @@ class TreeTab(Layout):
         ("border_width", 2, "Width of the border"),
         ("vspace", 2, "Space between tabs"),
         ("level_shift", 8, "Shift for children tabs"),
-        ("font", "Arial", "Font"),
+        ("font", "sans", "Font"),
         ("fontsize", 14, "Font pixel size."),
         ("fontshadow", None, "font shadow color, default is None (no shadow)"),
         ("section_fontsize", 11, "Font pixel size of section label"),
@@ -338,9 +369,9 @@ class TreeTab(Layout):
         ("section_padding", 4, "Bottom of margin section label"),
         ("section_left", 4, "Left margin of section label"),
         ("panel_width", 150, "Width of the left panel"),
-        ("sections", ['Default'], "Foreground color of inactive tab"),
-        ("name", "treetab", "Name of this layout."),
+        ("sections", ["Default"], "Foreground color of inactive tab"),
         ("previous_on_rm", False, "Focus previous window on close instead of first."),
+        ("place_right", False, "Place the tab panel on the right side"),
     ]
 
     def __init__(self, **config):
@@ -360,9 +391,6 @@ class TreeTab(Layout):
         c._tree = Root(self.sections)
         return c
 
-    def _get_window(self):
-        return self._focused
-
     def focus(self, win):
         self._focused = win
 
@@ -415,63 +443,103 @@ class TreeTab(Layout):
         del self._nodes[win]
         self.draw_panel()
 
-    def _create_panel(self):
-        self._panel = window.Internal.create(
-            self.group.qtile,
-            0,
-            0,
-            self.panel_width,
-            100
+    def _create_panel(self, screen_rect):
+        self._panel = self.group.qtile.core.create_internal(
+            screen_rect.x, screen_rect.y, self.panel_width, 100
         )
-        self._create_drawer()
-        self._panel.handle_Expose = self._panel_Expose
-        self._panel.handle_ButtonPress = self._panel_ButtonPress
-        self.group.qtile.windowMap[self._panel.window.wid] = self._panel
-        hook.subscribe.window_name_change(self.draw_panel)
+        self._create_drawer(screen_rect)
+        self._panel.process_window_expose = self.draw_panel
+        self._panel.process_button_click = self.process_button_click
+        hook.subscribe.client_name_updated(self.draw_panel)
         hook.subscribe.focus_change(self.draw_panel)
 
-    def _panel_Expose(self, e):
-        self.draw_panel()
-
-    def draw_panel(self):
+    def draw_panel(self, *args):
         if not self._panel:
             return
         self._drawer.clear(self.bg_color)
         self._tree.draw(self, 0)
         self._drawer.draw(offsetx=0, width=self.panel_width)
 
-    def _panel_ButtonPress(self, event):
-        node = self._tree.button_press(event.event_x, event.event_y)
+    def process_button_click(self, x, y, _buttom):
+        node = self._tree.button_press(x, y)
         if node:
             self.group.focus(node.window, False)
 
-    def configure(self, client, screen):
+    def configure(self, client, screen_rect):
         if self._nodes and client is self._focused:
             client.place(
-                screen.x, screen.y,
-                screen.width, screen.height,
-                0,
-                None
+                screen_rect.x, screen_rect.y, screen_rect.width, screen_rect.height, 0, None
             )
             client.unhide()
         else:
             client.hide()
 
     def finalize(self):
+        if self._panel:
+            self._panel.kill()
         Layout.finalize(self)
         if self._drawer is not None:
             self._drawer.finalize()
 
+    def get_windows(self):
+        clients = []
+        for section in self._tree.children:
+            for window in section.children:
+                clients.append(window.window)
+        return clients
+
     def info(self):
+        def show_section_tree(root):
+            """
+            Show a section tree in a nested list, whose every element has the form:
+            `[root, [subtrees]]`.
+
+            For `[root, [subtrees]]`, The first element is the root node, and the second
+            is its a list of its subtrees.  For example, a section with below windows
+            hierarchy on the panel:
+
+            - a
+              - d
+                - e
+              - f
+            - b
+              - g
+              - h
+            - c
+
+            will return [
+                         [a,
+                           [d, [e]],
+                           [f]],
+                         [b, [g], [h]],
+                         [c],
+                        ]
+            """
+            tree = []
+            if isinstance(root, Window):
+                tree.append(root.window.name)
+            if root.expanded and root.children:
+                for child in root.children:
+                    tree.append(show_section_tree(child))
+            return tree
+
         d = Layout.info(self)
-        d["clients"] = [x.name for x in self._nodes]
+        d["clients"] = sorted([x.name for x in self._nodes])
         d["sections"] = [x.title for x in self._tree.children]
+
+        trees = {}
+        for section in self._tree.children:
+            trees[section.title] = show_section_tree(section)
+        d["client_trees"] = trees
         return d
 
-    def show(self, screen):
+    def show(self, screen_rect):
         if not self._panel:
-            self._create_panel()
-        panel, body = screen.hsplit(self.panel_width)
+            self._create_panel(screen_rect)
+        if self.place_right:
+            body, panel = screen_rect.hsplit(screen_rect.width - self.panel_width)
+        else:
+            panel, body = screen_rect.hsplit(self.panel_width)
         self._resize_panel(panel)
         self._panel.unhide()
 
@@ -545,7 +613,7 @@ class TreeTab(Layout):
         self.draw_panel()
 
     def cmd_del_section(self, name):
-        """Add named section to tree"""
+        """Remove named section from tree"""
         self._tree.del_section(name)
         self.draw_panel()
 
@@ -582,9 +650,9 @@ class TreeTab(Layout):
 
         Parameters
         ==========
-        sorter : function with single arg returning string
+        sorter: function with single arg returning string
             returns name of the section where window should be
-        create_sections :
+        create_sections:
             if this parameter is True (default), if sorter returns unknown
             section name it will be created dynamically
         """
@@ -631,42 +699,35 @@ class TreeTab(Layout):
 
     def cmd_increase_ratio(self):
         self.panel_width += 10
-        self.group.layoutAll()
+        self.group.layout_all()
 
     def cmd_decrease_ratio(self):
         self.panel_width -= 10
-        self.group.layoutAll()
+        self.group.layout_all()
 
-    def _create_drawer(self):
+    def _create_drawer(self, screen_rect):
         if self._drawer is None:
-            self._drawer = drawer.Drawer(
-                self.group.qtile,
-                self._panel.window.wid,
+            self._drawer = self._panel.create_drawer(
                 self.panel_width,
-                self.group.screen.dheight
+                screen_rect.height,
             )
         self._drawer.clear(self.bg_color)
         self._layout = self._drawer.textlayout(
-            "",
-            "ffffff",
-            self.font,
-            self.fontsize,
-            self.fontshadow,
-            wrap=False
+            "", "ffffff", self.font, self.fontsize, self.fontshadow, wrap=False
         )
 
-    def layout(self, windows, screen):
-        panel, body = screen.hsplit(self.panel_width)
+    def layout(self, windows, screen_rect):
+        if self.place_right:
+            body, panel = screen_rect.hsplit(screen_rect.width - self.panel_width)
+        else:
+            panel, body = screen_rect.hsplit(self.panel_width)
         self._resize_panel(panel)
         Layout.layout(self, windows, body)
 
-    def _resize_panel(self, rect):
+    def _resize_panel(self, screen_rect):
         if self._panel:
             self._panel.place(
-                rect.x, rect.y,
-                rect.width, rect.height,
-                0,
-                None
+                screen_rect.x, screen_rect.y, screen_rect.width, screen_rect.height, 0, None
             )
-            self._create_drawer()
+            self._create_drawer(screen_rect)
             self.draw_panel()
