@@ -29,11 +29,16 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+import asyncio
+import copy
+import math
 import subprocess
 from typing import Any, List, Tuple
 
-from libqtile import bar, configurable, confreader, drawer
-from libqtile.command.base import CommandError, CommandObject
+from libqtile import bar, configurable, confreader
+from libqtile.command import interface
+from libqtile.command.base import CommandError, CommandObject, ItemT
+from libqtile.lazy import LazyCall
 from libqtile.log_utils import logger
 
 
@@ -71,10 +76,10 @@ class _Orientations(int):
         return self.doc
 
 
-ORIENTATION_NONE = _Orientations(0, 'none')
-ORIENTATION_HORIZONTAL = _Orientations(1, 'horizontal only')
-ORIENTATION_VERTICAL = _Orientations(2, 'vertical only')
-ORIENTATION_BOTH = _Orientations(3, 'horizontal and vertical')
+ORIENTATION_NONE = _Orientations(0, "none")
+ORIENTATION_HORIZONTAL = _Orientations(1, "horizontal only")
+ORIENTATION_VERTICAL = _Orientations(2, "vertical only")
+ORIENTATION_BOTH = _Orientations(3, "horizontal and vertical")
 
 
 class _Widget(CommandObject, configurable.Configurable):
@@ -91,9 +96,12 @@ class _Widget(CommandObject, configurable.Configurable):
     have been configured.
 
     Callback functions can be assigned to button presses by passing a dict to the
-    'callbacks' kwarg. No arguments are passed to the callback function so, if
+    'callbacks' kwarg. No arguments are passed to the function so, if
     you need access to the qtile object, it needs to be imported into your code.
 
+    ``lazy`` functions can also be passed as callback functions and can be used in
+    the same was as keybindings.
+
     For example:
 
     .. code-block:: python
@@ -103,23 +111,32 @@ class _Widget(CommandObject, configurable.Configurable):
         def open_calendar():
             qtile.cmd_spawn('gsimplecal next_month')
 
-        clock = widget.Clock(mouse_callbacks={'Button1': open_calendar})
+        clock = widget.Clock(
+            mouse_callbacks={
+                'Button1': open_calendar,
+                'Button3': lazy.spawn('gsimplecal prev_month')
+            }
+        )
 
     When the clock widget receives a click with button 1, the ``open_calendar`` function
-    will be executed. Callbacks can be assigned to other buttons by adding more entries
-    to the passed dictionary.
+    will be executed.
     """
+
     orientations = ORIENTATION_BOTH
-    offsetx = None
-    offsety = None
+    offsetx: int = 0
+    offsety: int = 0
     defaults = [
         ("background", None, "Widget background color"),
-        ("mouse_callbacks", {}, "Dict of mouse button press callback functions."),
+        (
+            "mouse_callbacks",
+            {},
+            "Dict of mouse button press callback functions. Acceps functions and ``lazy`` calls.",
+        ),
     ]  # type: List[Tuple[str, Any, str]]
 
     def __init__(self, length, **config):
         """
-            length: bar.STRETCH, bar.CALCULATED, or a specified length.
+        length: bar.STRETCH, bar.CALCULATED, or a specified length.
         """
         CommandObject.__init__(self)
         self.name = self.__class__.__name__.lower()
@@ -132,11 +149,14 @@ class _Widget(CommandObject, configurable.Configurable):
         if length in (bar.CALCULATED, bar.STRETCH):
             self.length_type = length
             self.length = 0
-        else:
-            assert isinstance(length, int)
+        elif isinstance(length, int):
             self.length_type = bar.STATIC
             self.length = length
+        else:
+            raise confreader.ConfigError("Widget width must be an int")
+
         self.configured = False
+        self._futures: List[asyncio.TimerHandle] = []
 
     @property
     def length(self):
@@ -152,12 +172,12 @@ class _Widget(CommandObject, configurable.Configurable):
     def width(self):
         if self.bar.horizontal:
             return self.length
-        return self.bar.size
+        return self.bar.size - (self.bar.border_width[1] + self.bar.border_width[3])
 
     @property
     def height(self):
         if self.bar.horizontal:
-            return self.bar.size
+            return self.bar.size - (self.bar.border_width[0] + self.bar.border_width[2])
         return self.length
 
     @property
@@ -166,52 +186,54 @@ class _Widget(CommandObject, configurable.Configurable):
             return self.offsetx
         return self.offsety
 
-    @property
-    def win(self):
-        return self.bar.window.window
-
-    # Do not start the name with "test", or nosetests will try to test it
-    # directly (prepend an underscore instead)
     def _test_orientation_compatibility(self, horizontal):
         if horizontal:
             if not self.orientations & ORIENTATION_HORIZONTAL:
                 raise confreader.ConfigError(
-                    self.__class__.__name__ +
-                    " is not compatible with the orientation of the bar."
+                    self.__class__.__name__
+                    + " is not compatible with the orientation of the bar."
                 )
         elif not self.orientations & ORIENTATION_VERTICAL:
             raise confreader.ConfigError(
-                self.__class__.__name__ +
-                " is not compatible with the orientation of the bar."
+                self.__class__.__name__ + " is not compatible with the orientation of the bar."
             )
 
     def timer_setup(self):
-        """ This is called exactly once, after the widget has been configured
-        and timers are available to be set up. """
+        """This is called exactly once, after the widget has been configured
+        and timers are available to be set up."""
         pass
 
     def _configure(self, qtile, bar):
+        self._test_orientation_compatibility(bar.horizontal)
+
         self.qtile = qtile
         self.bar = bar
-        self.drawer = drawer.Drawer(
-            qtile,
-            self.win.wid,
-            self.bar.width,
-            self.bar.height
-        )
+        self.drawer = bar.window.create_drawer(self.bar.width, self.bar.height)
         if not self.configured:
-            self.configured = True
             self.qtile.call_soon(self.timer_setup)
+            self.qtile.call_soon(asyncio.create_task, self._config_async())
+
+    async def _config_async(self):
+        """
+        This is called once when the main eventloop has started. this
+        happens after _configure has been run.
+
+        Widgets that need to use asyncio coroutines after this point may
+        wish to initialise the relevant code (e.g. connections to dbus
+        using dbus_next) here.
+        """
+        pass
 
     def finalize(self):
-        if hasattr(self, 'layout') and self.layout:
+        for future in self._futures:
+            future.cancel()
+        if hasattr(self, "layout") and self.layout:
             self.layout.finalize()
         self.drawer.finalize()
 
     def clear(self):
         self.drawer.set_source_rgb(self.bar.background)
-        self.drawer.fillrect(self.offsetx, self.offsety, self.width,
-                             self.height)
+        self.drawer.fillrect(self.offsetx, self.offsety, self.width, self.height)
 
     def info(self):
         return dict(
@@ -228,79 +250,105 @@ class _Widget(CommandObject, configurable.Configurable):
         self.mouse_callbacks = defaults
 
     def button_press(self, x, y, button):
-        name = 'Button{0}'.format(button)
+        name = "Button{0}".format(button)
         if name in self.mouse_callbacks:
-            self.mouse_callbacks[name]()
+            cmd = self.mouse_callbacks[name]
+            if isinstance(cmd, LazyCall):
+                if cmd.check(self.qtile):
+                    status, val = self.qtile.server.call(
+                        (cmd.selectors, cmd.name, cmd.args, cmd.kwargs)
+                    )
+                    if status in (interface.ERROR, interface.EXCEPTION):
+                        logger.error("Mouse callback command error %s: %s" % (cmd.name, val))
+            else:
+                cmd()
 
     def button_release(self, x, y, button):
         pass
 
     def get(self, q, name):
         """
-            Utility function for quick retrieval of a widget by name.
+        Utility function for quick retrieval of a widget by name.
         """
         w = q.widgets_map.get(name)
         if not w:
             raise CommandError("No such widget: %s" % name)
         return w
 
-    def _items(self, name):
+    def _items(self, name: str) -> ItemT:
         if name == "bar":
-            return (True, None)
+            return True, []
+        elif name == "screen":
+            return True, []
+        return None
 
     def _select(self, name, sel):
         if name == "bar":
             return self.bar
+        elif name == "screen":
+            return self.bar.screen
 
     def cmd_info(self):
         """
-            Info for this object.
+        Info for this object.
         """
         return self.info()
 
     def draw(self):
         """
-            Method that draws the widget. You may call this explicitly to
-            redraw the widget, but only if the length of the widget hasn't
-            changed. If it has, you must call bar.draw instead.
+        Method that draws the widget. You may call this explicitly to
+        redraw the widget, but only if the length of the widget hasn't
+        changed. If it has, you must call bar.draw instead.
         """
         raise NotImplementedError
 
     def calculate_length(self):
         """
-            Must be implemented if the widget can take CALCULATED for length.
-            It must return the width of the widget if it's installed in a
-            horizontal bar; it must return the height of the widget if it's
-            installed in a vertical bar. Usually you will test the orientation
-            of the bar with 'self.bar.horizontal'.
+        Must be implemented if the widget can take CALCULATED for length.
+        It must return the width of the widget if it's installed in a
+        horizontal bar; it must return the height of the widget if it's
+        installed in a vertical bar. Usually you will test the orientation
+        of the bar with 'self.bar.horizontal'.
         """
         raise NotImplementedError
 
     def timeout_add(self, seconds, method, method_args=()):
         """
-            This method calls either ``.call_later`` with given arguments.
+        This method calls ``.call_later`` with given arguments.
         """
-        return self.qtile.call_later(seconds, self._wrapper, method,
-                                     *method_args)
+        future = self.qtile.call_later(seconds, self._wrapper, method, *method_args)
+
+        self._futures.append(future)
+        return future
 
     def call_process(self, command, **kwargs):
         """
-            This method uses `subprocess.check_output` to run the given command
-            and return the string from stdout, which is decoded when using
-            Python 3.
+        This method uses `subprocess.check_output` to run the given command
+        and return the string from stdout, which is decoded when using
+        Python 3.
         """
-        output = subprocess.check_output(command, **kwargs)
-        output = output.decode()
-        return output
+        return subprocess.check_output(command, **kwargs, encoding="utf-8")
+
+    def _remove_dead_timers(self):
+        """Remove completed and cancelled timers from the list."""
+        self._futures = [
+            timer
+            for timer in self._futures
+            if not (timer.cancelled() or timer.when() < self.qtile._eventloop.time())
+        ]
 
     def _wrapper(self, method, *method_args):
+        self._remove_dead_timers()
         try:
             method(*method_args)
         except:  # noqa: E722
-            logger.exception('got exception from widget timer')
+            logger.exception("got exception from widget timer")
 
     def create_mirror(self):
-        return Mirror(self)
+        return Mirror(self, background=self.background)
+
+    def clone(self):
+        return copy.copy(self)
 
     def mouse_enter(self, x, y):
         pass
@@ -314,28 +362,26 @@ UNSPECIFIED = bar.Obj("UNSPECIFIED")
 
 class _TextBox(_Widget):
     """
-        Base class for widgets that are just boxes containing text.
+    Base class for widgets that are just boxes containing text.
     """
-    orientations = ORIENTATION_HORIZONTAL
+
+    orientations = ORIENTATION_BOTH
     defaults = [
         ("font", "sans", "Default font"),
         ("fontsize", None, "Font size. Calculated if None."),
         ("padding", None, "Padding. Calculated if None."),
         ("foreground", "ffffff", "Foreground colour"),
-        (
-            "fontshadow",
-            None,
-            "font shadow color, default is None(no shadow)"
-        ),
+        ("fontshadow", None, "font shadow color, default is None(no shadow)"),
         ("markup", True, "Whether or not to use pango markup"),
-        ("fmt", "{}", "How to format the text")
+        ("fmt", "{}", "How to format the text"),
+        ("max_chars", 0, "Maximum number of characters to display in widget."),
     ]  # type: List[Tuple[str, Any, str]]
 
     def __init__(self, text=" ", width=bar.CALCULATED, **config):
         self.layout = None
         _Widget.__init__(self, width, **config)
-        self._text = text
         self.add_defaults(_TextBox.defaults)
+        self.text = text
 
     @property
     def text(self):
@@ -343,6 +389,8 @@ class _TextBox(_Widget):
 
     @text.setter
     def text(self, value):
+        if len(value) > self.max_chars > 0:
+            value = value[: self.max_chars] + "…"
         self._text = value
         if self.layout:
             self.layout.text = self.formatted_text
@@ -403,29 +451,53 @@ class _TextBox(_Widget):
 
     def calculate_length(self):
         if self.text:
-            return min(
-                self.layout.width,
-                self.bar.width
-            ) + self.actual_padding * 2
+            if self.bar.horizontal:
+                return min(self.layout.width, self.bar.width) + self.actual_padding * 2
+            else:
+                return min(self.layout.width, self.bar.height) + self.actual_padding * 2
         else:
             return 0
 
+    def can_draw(self):
+        can_draw = (
+            self.layout is not None and not self.layout.finalized() and self.offsetx is not None
+        )  # if the bar hasn't placed us yet
+        return can_draw
+
     def draw(self):
-        # if the bar hasn't placed us yet
-        if self.offsetx is None:
+        if not self.can_draw():
             return
         self.drawer.clear(self.background or self.bar.background)
-        self.layout.draw(
-            self.actual_padding or 0,
-            int(self.bar.height / 2.0 - self.layout.height / 2.0) + 1
-        )
-        self.drawer.draw(offsetx=self.offsetx, width=self.width)
+        if self.bar.horizontal:
+            self.layout.draw(
+                self.actual_padding or 0,
+                int(self.bar.height / 2.0 - self.layout.height / 2.0) + 1,
+            )
+        else:
+            # We need to do some transformations for vertical bars.
+            self.drawer.ctx.save()
 
-    def cmd_set_font(self, font=UNSPECIFIED, fontsize=UNSPECIFIED,
-                     fontshadow=UNSPECIFIED):
+            # Left bar reads bottom to top
+            if self.bar.screen.left is self.bar:
+                self.drawer.ctx.rotate(-90 * math.pi / 180.0)
+                self.drawer.ctx.translate(-self.length, 0)
+
+            # Right bar is top to bottom
+            else:
+                self.drawer.ctx.translate(self.bar.width, 0)
+                self.drawer.ctx.rotate(90 * math.pi / 180.0)
+
+            self.layout.draw(
+                self.actual_padding or 0, int(self.bar.width / 2.0 - self.layout.height / 2.0) + 1
+            )
+            self.drawer.ctx.restore()
+
+        self.drawer.draw(offsetx=self.offsetx, offsety=self.offsety, width=self.width)
+
+    def cmd_set_font(self, font=UNSPECIFIED, fontsize=UNSPECIFIED, fontshadow=UNSPECIFIED):
         """
-            Change the font used by this widget. If font is None, the current
-            font is used.
+        Change the font used by this widget. If font is None, the current
+        font is used.
         """
         if font is not UNSPECIFIED:
             self.font = font
@@ -437,22 +509,42 @@ class _TextBox(_Widget):
 
     def info(self):
         d = _Widget.info(self)
-        d['foreground'] = self.foreground
-        d['text'] = self.formatted_text
+        d["foreground"] = self.foreground
+        d["text"] = self.formatted_text
         return d
 
+    def update(self, text):
+        if self.text == text:
+            return
+        if text is None:
+            text = ""
+
+        old_width = self.layout.width
+        self.text = text
+
+        # If our width hasn't changed, we just draw ourselves. Otherwise,
+        # we draw the whole bar.
+        if self.layout.width == old_width:
+            self.draw()
+        else:
+            self.bar.draw()
+
 
 class InLoopPollText(_TextBox):
-    """ A common interface for polling some 'fast' information, munging it, and
+    """A common interface for polling some 'fast' information, munging it, and
     rendering the result in a text box. You probably want to use
     ThreadPoolText instead.
 
     ('fast' here means that this runs /in/ the event loop, so don't block! If
-    you want to run something nontrivial, use ThreadedPollWidget.) """
+    you want to run something nontrivial, use ThreadedPollWidget.)"""
 
     defaults = [
-        ("update_interval", 600, "Update interval in seconds, if none, the "
-            "widget updates whenever the event loop is idle."),
+        (
+            "update_interval",
+            600,
+            "Update interval in seconds, if none, the "
+            "widget updates whenever the event loop is idle.",
+        ),
     ]  # type: List[Tuple[str, Any, str]]
 
     def __init__(self, default_text="N/A", width=bar.CALCULATED, **config):
@@ -483,26 +575,15 @@ class InLoopPollText(_TextBox):
         _TextBox.button_press(self, x, y, button)
 
     def poll(self):
-        return 'N/A'
+        return "N/A"
 
     def tick(self):
         text = self.poll()
         self.update(text)
 
-    def update(self, text):
-        old_width = self.layout.width
-        if self.text != text:
-            self.text = text
-            # If our width hasn't changed, we just draw ourselves. Otherwise,
-            # we draw the whole bar.
-            if self.layout.width == old_width:
-                self.draw()
-            else:
-                self.bar.draw()
-
 
 class ThreadPoolText(_TextBox):
-    """ A common interface for wrapping blocking events which when triggered
+    """A common interface for wrapping blocking events which when triggered
     will update a textbox.
 
     The poll method is intended to wrap a blocking function which may take
@@ -512,9 +593,13 @@ class ThreadPoolText(_TextBox):
 
     param: text - Initial text to display.
     """
+
     defaults = [
-        ("update_interval", 600, "Update interval in seconds, if none, the "
-            "widget updates whenever it's done'."),
+        (
+            "update_interval",
+            600,
+            "Update interval in seconds, if none, the " "widget updates whenever it's done.",
+        ),
     ]  # type: List[Tuple[str, Any, str]]
 
     def __init__(self, text, **config):
@@ -527,7 +612,7 @@ class ThreadPoolText(_TextBox):
                 result = future.result()
             except Exception:
                 result = None
-                logger.exception('poll() raised exceptions, not rescheduling')
+                logger.exception("poll() raised exceptions, not rescheduling")
 
             if result is not None:
                 try:
@@ -539,28 +624,17 @@ class ThreadPoolText(_TextBox):
                         self.timer_setup()
 
                 except Exception:
-                    logger.exception('Failed to reschedule.')
+                    logger.exception("Failed to reschedule.")
             else:
-                logger.warning('poll() returned None, not rescheduling')
+                logger.warning("poll() returned None, not rescheduling")
 
-        future = self.qtile.run_in_executor(self.poll)
-        future.add_done_callback(on_done)
-
-    def update(self, text):
-        old_width = self.layout.width
-        if self.text == text:
-            return
-
-        self.text = text
-
-        if self.layout.width == old_width:
-            self.draw()
-        else:
-            self.bar.draw()
+        self.future = self.qtile.run_in_executor(self.poll)
+        self.future.add_done_callback(on_done)
 
     def poll(self):
         pass
 
+
 # these two classes below look SUSPICIOUSLY similar
 
 
@@ -578,8 +652,8 @@ class PaddingMixin(configurable.Configurable):
         ("padding_y", None, "Y Padding. Overrides 'padding' if set"),
     ]  # type: List[Tuple[str, Any, str]]
 
-    padding_x = configurable.ExtraFallback('padding_x', 'padding')
-    padding_y = configurable.ExtraFallback('padding_y', 'padding')
+    padding_x = configurable.ExtraFallback("padding_x", "padding")
+    padding_y = configurable.ExtraFallback("padding_y", "padding")
 
 
 class MarginMixin(configurable.Configurable):
@@ -596,17 +670,47 @@ class MarginMixin(configurable.Configurable):
         ("margin_y", None, "Y Margin. Overrides 'margin' if set"),
     ]  # type: List[Tuple[str, Any, str]]
 
-    margin_x = configurable.ExtraFallback('margin_x', 'margin')
-    margin_y = configurable.ExtraFallback('margin_y', 'margin')
+    margin_x = configurable.ExtraFallback("margin_x", "margin")
+    margin_y = configurable.ExtraFallback("margin_y", "margin")
 
 
 class Mirror(_Widget):
-    def __init__(self, reflection):
-        _Widget.__init__(self, reflection.length)
+    """
+    A widget for showing the same widget content in more than one place, for
+    instance, on bars across multiple screens.
+
+    You don't need to use it directly; instead, just instantiate your widget
+    once and hand it in to multiple bars. For instance::
+
+        cpu = widget.CPUGraph()
+        clock = widget.Clock()
+
+        screens = [
+            Screen(top=bar.Bar([widget.GroupBox(), cpu, clock])),
+            Screen(top=bar.Bar([widget.GroupBox(), cpu, clock])),
+        ]
+
+    Widgets can be passed to more than one bar, so that there don't need to be
+    any duplicates executing the same code all the time, and they'll always be
+    visually identical.
+
+    This works for all widgets that use `drawers` (and nothing else) to display
+    their contents. Currently, this is all widgets except for `Systray`.
+    """
+
+    def __init__(self, reflection, **config):
+        _Widget.__init__(self, reflection.length, **config)
         reflection.draw = self.hook(reflection.draw)
         self.reflects = reflection
         self._length = 0
 
+    def _configure(self, qtile, bar):
+        _Widget._configure(self, qtile, bar)
+        self.reflects.drawer.add_mirror(self.drawer)
+        # We need to fill the background once before `draw` is called so, if
+        # there's no reflection, the mirror matches its parent bar.
+        self.drawer.clear(self.background or self.bar.background)
+
     @property
     def length(self):
         return self.reflects.length
@@ -619,6 +723,7 @@ class Mirror(_Widget):
         def _():
             draw()
             self.draw()
+
         return _
 
     def draw(self):
@@ -626,8 +731,13 @@ class Mirror(_Widget):
             self._length = self.length
             self.bar.draw()
         else:
-            self.reflects.drawer.paint_to(self.drawer)
-            self.drawer.draw(offsetx=self.offset, width=self.width)
+            # We only update the mirror's drawer if the parent widget has
+            # contents in its RecordingSurface. If this is False then the widget
+            # wil just show the existing drawer contents.
+            if self.reflects.drawer.needs_update:
+                self.drawer.clear(self.background or self.bar.background)
+                self.reflects.drawer.paint_to(self.drawer)
+            self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.width)
 
     def button_press(self, x, y, button):
         self.reflects.button_press(x, y, button)
