@@ -31,16 +31,18 @@
 # SOFTWARE.
 
 import itertools
+from functools import partial
+from typing import Any, List, Tuple
 
-from .. import bar, hook
-from . import base
+from libqtile import bar, hook
+from libqtile.widget import base
 
 
 class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
     defaults = [
         ("borderwidth", 3, "Current group border width"),
-        ("center_aligned", False, "center-aligned group box"),
-    ]
+        ("center_aligned", True, "center-aligned group box"),
+    ]  # type: List[Tuple[str, Any, str]]
 
     def __init__(self, **config):
         base._TextBox.__init__(self, width=bar.CALCULATED, **config)
@@ -50,9 +52,7 @@ class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
 
     def box_width(self, groups):
         width, _ = self.drawer.max_layout_size(
-            [i.name for i in groups],
-            self.font,
-            self.fontsize
+            [i.label for i in groups], self.font, self.fontsize
         )
         return width + self.padding_x * 2 + self.borderwidth * 2
 
@@ -60,22 +60,18 @@ class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
         base._Widget._configure(self, qtile, bar)
 
         if self.fontsize is None:
-            calc = self.bar.height - self.margin_y * 2 - \
-                self.borderwidth * 2 - self.padding_y * 2
+            calc = self.bar.height - self.margin_y * 2 - self.borderwidth * 2 - self.padding_y * 2
             self.fontsize = max(calc, 1)
 
         self.layout = self.drawer.textlayout(
-            "",
-            "ffffff",
-            self.font,
-            self.fontsize,
-            self.fontshadow
+            "", "ffffff", self.font, self.fontsize, self.fontshadow
         )
         self.setup_hooks()
 
     def setup_hooks(self):
         def hook_response(*args, **kwargs):
             self.bar.draw()
+
         hook.subscribe.client_managed(hook_response)
         hook.subscribe.client_urgent_hint_changed(hook_response)
         hook.subscribe.client_killed(hook_response)
@@ -84,8 +80,19 @@ class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
         hook.subscribe.current_screen_change(hook_response)
         hook.subscribe.changegroup(hook_response)
 
-    def drawbox(self, offset, text, bordercolor, textcolor, highlight_color=None,
-                width=None, rounded=False, block=False, line=False, highlighted=False):
+    def drawbox(
+        self,
+        offset,
+        text,
+        bordercolor,
+        textcolor,
+        highlight_color=None,
+        width=None,
+        rounded=False,
+        block=False,
+        line=False,
+        highlighted=False,
+    ):
         self.layout.text = text
         self.layout.font_family = self.font
         self.layout.font_size = self.fontsize
@@ -95,24 +102,29 @@ class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
         if line:
             pad_y = [
                 (self.bar.height - self.layout.height - self.borderwidth) / 2,
-                (self.bar.height - self.layout.height + self.borderwidth) / 2
+                (self.bar.height - self.layout.height + self.borderwidth) / 2,
             ]
         else:
             pad_y = self.padding_y
-        framed = self.layout.framed(
-            self.borderwidth,
-            bordercolor,
-            0,
-            pad_y,
-            highlight_color
-        )
+
+        if bordercolor is None:
+            # border colour is set to None when we don't want to draw a border at all
+            # Rather than dealing with alpha blending issues, we just set border width
+            # to 0.
+            border_width = 0
+            framecolor = self.background or self.bar.background
+        else:
+            border_width = self.borderwidth
+            framecolor = bordercolor
+
+        framed = self.layout.framed(border_width, framecolor, 0, pad_y, highlight_color)
         y = self.margin_y
         if self.center_aligned:
             for t in base.MarginMixin.defaults:
-                if t[0] == 'margin':
+                if t[0] == "margin":
                     y += (self.bar.height - framed.height) / 2 - t[1]
                     break
-        if block:
+        if block and bordercolor is not None:
             framed.draw_fill(offset, y, rounded)
         elif line:
             framed.draw_line(offset, y, highlighted)
@@ -122,6 +134,7 @@ class _GroupBase(base._TextBox, base.PaddingMixin, base.MarginMixin):
 
 class AGroupBox(_GroupBase):
     """A widget that graphically displays the current group"""
+
     orientations = base.ORIENTATION_HORIZONTAL
     defaults = [("border", "000000", "group box border color")]
 
@@ -129,86 +142,88 @@ class AGroupBox(_GroupBase):
         _GroupBase.__init__(self, **config)
         self.add_defaults(AGroupBox.defaults)
 
-    def button_press(self, x, y, button):
-        self.bar.screen.cmd_next_group()
+    def _configure(self, qtile, bar):
+        _GroupBase._configure(self, qtile, bar)
+        self.add_callbacks({"Button1": partial(self.bar.screen.cmd_next_group, warp=False)})
 
     def calculate_length(self):
         return self.box_width(self.qtile.groups) + self.margin_x * 2
 
     def draw(self):
         self.drawer.clear(self.background or self.bar.background)
-        e = next(
-            i for i in self.qtile.groups
-            if i.name == self.bar.screen.group.name
-        )
+        e = next(i for i in self.qtile.groups if i.name == self.bar.screen.group.name)
         self.drawbox(self.margin_x, e.name, self.border, self.foreground)
-        self.drawer.draw(offsetx=self.offset, width=self.width)
+        self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.width)
 
 
 class GroupBox(_GroupBase):
-    """A widget that graphically displays the current group"""
+    """
+    A widget that graphically displays the current group.
+    All groups are displayed by their label.
+    If the label of a group is the empty string that group will not be displayed.
+    """
+
     orientations = base.ORIENTATION_HORIZONTAL
     defaults = [
+        ("block_highlight_text_color", None, "Selected group font colour"),
         ("active", "FFFFFF", "Active group font colour"),
         ("inactive", "404040", "Inactive group font colour"),
         (
             "highlight_method",
             "border",
             "Method of highlighting ('border', 'block', 'text', or 'line')"
-            "Uses \*_border color settings"
+            "Uses `*_border` color settings",
         ),
         ("rounded", True, "To round or not to round box borders"),
         (
             "this_current_screen_border",
             "215578",
-            "Border or line colour for group on this screen when focused."
+            "Border or line colour for group on this screen when focused.",
         ),
         (
             "this_screen_border",
             "215578",
-            "Border or line colour for group on this screen when unfocused."
+            "Border or line colour for group on this screen when unfocused.",
         ),
         (
             "other_current_screen_border",
             "404040",
-            "Border or line colour for group on other screen when focused."
+            "Border or line colour for group on other screen when focused.",
         ),
         (
             "other_screen_border",
             "404040",
-            "Border or line colour for group on other screen when unfocused."
+            "Border or line colour for group on other screen when unfocused.",
         ),
         (
             "highlight_color",
             ["000000", "282828"],
-            "Active group highlight color when using 'line' highlight method."
+            "Active group highlight color when using 'line' highlight method.",
         ),
         (
             "urgent_alert_method",
             "border",
             "Method for alerting you of WM urgent "
-            "hints (one of 'border', 'text', 'block', or 'line')"
+            "hints (one of 'border', 'text', 'block', or 'line')",
         ),
         ("urgent_text", "FF0000", "Urgent group font color"),
         ("urgent_border", "FF0000", "Urgent border or line color"),
-        (
-            "disable_drag",
-            False,
-            "Disable dragging and dropping of group names on widget"
-        ),
+        ("disable_drag", False, "Disable dragging and dropping of group names on widget"),
         ("invert_mouse_wheel", False, "Whether to invert mouse wheel group movement"),
         ("use_mouse_wheel", True, "Whether to use mouse wheel events"),
         (
             "visible_groups",
             None,
-            "Groups that will be visible "
-            "(if set to None or [], all groups will be visible)"
+            "Groups that will be visible. "
+            "If set to None or [], all groups will be visible."
+            "Visible groups are identified by name not by their displayed label.",
         ),
         (
-            "spacing",
-            None,
-            "Spacing between groups"
-            "(if set to None, will be equal to margin_x)")
+            "hide_unused",
+            False,
+            "Hide groups that have no windows and that are not displayed on any screen.",
+        ),
+        ("spacing", None, "Spacing between groups" "(if set to None, will be equal to margin_x)"),
     ]
 
     def __init__(self, **config):
@@ -217,54 +232,95 @@ class GroupBox(_GroupBase):
         if self.spacing is None:
             self.spacing = self.margin_x
         self.clicked = None
+        self.click = None
+
+        default_callbacks = {"Button1": self.select_group}
+        if self.use_mouse_wheel:
+            default_callbacks.update(
+                {
+                    "Button5" if self.invert_mouse_wheel else "Button4": self.prev_group,
+                    "Button4" if self.invert_mouse_wheel else "Button5": self.next_group,
+                }
+            )
+        self.add_callbacks(default_callbacks)
 
     @property
     def groups(self):
-        return self.qtile.groups if not self.visible_groups else \
-            [g for g in self.qtile.groups if g.name in self.visible_groups]
+        """
+        returns list of visible groups.
+        The existing groups are filtered by the visible_groups attribute and
+        their label. Groups with an empty string as label are never contained.
+        Groups that are not named in visible_groups are not returned.
+        """
+        if self.hide_unused:
+            if self.visible_groups:
+                return [
+                    g
+                    for g in self.qtile.groups
+                    if g.label and (g.windows or g.screen) and g.name in self.visible_groups
+                ]
+            else:
+                return [g for g in self.qtile.groups if g.label and (g.windows or g.screen)]
+        else:
+            if self.visible_groups:
+                return [g for g in self.qtile.groups if g.label and g.name in self.visible_groups]
+            else:
+                return [g for g in self.qtile.groups if g.label]
 
-    def get_clicked_group(self, x, y):
+    def get_clicked_group(self):
         group = None
         new_width = self.margin_x - self.spacing / 2.0
         width = 0
         for g in self.groups:
             new_width += self.box_width([g]) + self.spacing
-            if width <= x <= new_width:
+            if width <= self.click <= new_width:
                 group = g
                 break
             width = new_width
         return group
 
     def button_press(self, x, y, button):
-        self.clicked = None
+        self.click = x
+        _GroupBase.button_press(self, x, y, button)
+
+    def next_group(self):
         group = None
-        curGroup = self.qtile.currentGroup
+        current_group = self.qtile.current_group
+        i = itertools.cycle(self.qtile.groups)
+        while next(i) != current_group:
+            pass
+        while group is None or group not in self.groups:
+            group = next(i)
+        self.go_to_group(group)
 
-        if button == (5 if not self.invert_mouse_wheel else 4):
-            if self.use_mouse_wheel:
-                i = itertools.cycle(self.qtile.groups)
-                while next(i) != curGroup:
-                    pass
-                while group is None or group not in self.groups:
-                    group = next(i)
-        elif button == (4 if not self.invert_mouse_wheel else 5):
-            if self.use_mouse_wheel:
-                i = itertools.cycle(reversed(self.qtile.groups))
-                while next(i) != curGroup:
-                    pass
-                while group is None or group not in self.groups:
-                    group = next(i)
-        else:
-            group = self.get_clicked_group(x, y)
-            if not self.disable_drag:
-                self.clicked = group
+    def prev_group(self):
+        group = None
+        current_group = self.qtile.current_group
+        i = itertools.cycle(reversed(self.qtile.groups))
+        while next(i) != current_group:
+            pass
+        while group is None or group not in self.groups:
+            group = next(i)
+        self.go_to_group(group)
 
+    def select_group(self):
+        self.clicked = None
+        group = self.get_clicked_group()
+        if not self.disable_drag:
+            self.clicked = group
+        self.go_to_group(group)
+
+    def go_to_group(self, group):
         if group:
-            self.bar.screen.setGroup(group)
+            if self.bar.screen.group != group or not self.disable_drag:
+                self.bar.screen.set_group(group, warp=False)
+            else:
+                self.bar.screen.toggle_group(group, warp=False)
 
     def button_release(self, x, y, button):
+        self.click = x
         if button not in (5, 4):
-            group = self.get_clicked_group(x, y)
+            group = self.get_clicked_group()
             if group and self.clicked:
                 group.cmd_switch_groups(self.clicked.name)
                 self.clicked = None
@@ -284,8 +340,8 @@ class GroupBox(_GroupBase):
         offset = self.margin_x
         for i, g in enumerate(self.groups):
             to_highlight = False
-            is_block = (self.highlight_method == 'block')
-            is_line = (self.highlight_method == 'line')
+            is_block = self.highlight_method == "block"
+            is_line = self.highlight_method == "line"
 
             bw = self.box_width([g])
 
@@ -297,34 +353,39 @@ class GroupBox(_GroupBase):
                 text_color = self.inactive
 
             if g.screen:
-                if self.highlight_method == 'text':
-                    border = self.bar.background
+                if self.highlight_method == "text":
+                    border = None
                     text_color = self.this_current_screen_border
                 else:
+                    if self.block_highlight_text_color:
+                        text_color = self.block_highlight_text_color
                     if self.bar.screen.group.name == g.name:
-                        if self.qtile.currentScreen == self.bar.screen:
+                        if self.qtile.current_screen == self.bar.screen:
                             border = self.this_current_screen_border
                             to_highlight = True
                         else:
                             border = self.this_screen_border
                     else:
-                        if self.qtile.currentScreen == g.screen:
+                        if self.qtile.current_screen == g.screen:
                             border = self.other_current_screen_border
                         else:
                             border = self.other_screen_border
-            elif self.group_has_urgent(g) and \
-                    self.urgent_alert_method in ('border', 'block', 'line'):
+            elif self.group_has_urgent(g) and self.urgent_alert_method in (
+                "border",
+                "block",
+                "line",
+            ):
                 border = self.urgent_border
-                if self.urgent_alert_method == 'block':
+                if self.urgent_alert_method == "block":
                     is_block = True
-                elif self.urgent_alert_method == 'line':
+                elif self.urgent_alert_method == "line":
                     is_line = True
             else:
-                border = self.background or self.bar.background
+                border = None
 
             self.drawbox(
                 offset,
-                g.name,
+                g.label,
                 border,
                 text_color,
                 highlight_color=self.highlight_color,
@@ -332,7 +393,7 @@ class GroupBox(_GroupBase):
                 rounded=self.rounded,
                 block=is_block,
                 line=is_line,
-                highlighted=to_highlight
+                highlighted=to_highlight,
             )
             offset += bw + self.spacing
-        self.drawer.draw(offsetx=self.offset, width=self.width)
+        self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.width)
