@@ -29,8 +29,7 @@
 def assert_focused(self, name):
     """Asserts that window with specified name is currently focused"""
     info = self.c.window.info()
-    assert info['name'] == name, 'Got {0!r}, expected {1!r}'.format(
-        info['name'], name)
+    assert info["name"] == name, "Got {0!r}, expected {1!r}".format(info["name"], name)
 
 
 def assert_dimensions(self, x, y, w, h, win=None):
@@ -38,10 +37,10 @@ def assert_dimensions(self, x, y, w, h, win=None):
     if win is None:
         win = self.c.window
     info = win.info()
-    assert info['x'] == x, info
-    assert info['y'] == y, info
-    assert info['width'] == w, info
-    assert info['height'] == h, info
+    assert info["x"] == x, info
+    assert info["y"] == y, info
+    assert info["width"] == w, info
+    assert info["height"] == h, info
 
 
 def assert_dimensions_fit(self, x, y, w, h, win=None):
@@ -49,10 +48,10 @@ def assert_dimensions_fit(self, x, y, w, h, win=None):
     if win is None:
         win = self.c.window
     info = win.info()
-    assert info['x'] >= x, info
-    assert info['y'] >= y, info
-    assert info['width'] <= w, info
-    assert info['height'] <= h, info
+    assert info["x"] >= x, info
+    assert info["y"] >= y, info
+    assert info["width"] <= w, info
+    assert info["height"] <= h, info
 
 
 def assert_focus_path(self, *names):
@@ -89,7 +88,7 @@ def assert_focus_path_unordered(self, *names):
     ordered_names = []
     while unordered_names:
         self.c.group.next_window()
-        wname = self.c.window.info()['name']
+        wname = self.c.window.info()["name"]
         assert wname in unordered_names
         unordered_names.remove(wname)
         ordered_names.append(wname)
