@@ -31,13 +31,14 @@ from test.layouts.layout_utils import (
     assert_focused,
 )
 
-
 try:
     # Check to see if we should skip tests using notifications on Wayland
     import gi
-    gi.require_version('Gtk', '3.0')
-    gi.require_version('GtkLayerShell', '0.1')
-    from gi.repository import GtkLayerShell
+
+    gi.require_version("Gtk", "3.0")
+    gi.require_version("GtkLayerShell", "0.1")
+    from gi.repository import GtkLayerShell  # noqa: F401
+
     has_wayland_notifications = True
 except (ImportError, ValueError):
     has_wayland_notifications = False
@@ -47,6 +48,7 @@ class AllLayoutsConfig(Config):
     """
     Ensure that all layouts behave consistently in some common scenarios.
     """
+
     groups = [
         libqtile.config.Group("a"),
         libqtile.config.Group("b"),
@@ -70,7 +72,7 @@ class AllLayoutsConfig(Config):
             else:
                 # Explicitly exclude the Slice layout, since it depends on
                 # other layouts (tested here) and has its own specific tests
-                if test and layout_name != 'Slice':
+                if test and layout_name != "Slice":
                     yield layout_name, layout_cls
 
     @classmethod
@@ -80,12 +82,13 @@ class AllLayoutsConfig(Config):
         Each configuration has only the tested layout (i.e. 1 item) in the
         'layouts' variable.
         """
-        return [type(layout_name, (cls, ), {'layouts': [layout_cls()]})
-                for layout_name, layout_cls in cls.iter_layouts()]
+        return [
+            type(layout_name, (cls,), {"layouts": [layout_cls()]})
+            for layout_name, layout_cls in cls.iter_layouts()
+        ]
 
 
 class AllDelegateLayoutsConfig(AllLayoutsConfig):
-
     @classmethod
     def generate(cls):
         """
@@ -95,11 +98,13 @@ class AllDelegateLayoutsConfig(AllLayoutsConfig):
         'layouts' variable.
         """
         return [
-            type(layout_name, (cls, ), {
-                'layouts': [
-                    layout.slice.Slice(
-                        wname='nevermatch', fallback=layout_cls())]})
-            for layout_name, layout_cls in cls.iter_layouts()]
+            type(
+                layout_name,
+                (cls,),
+                {"layouts": [layout.slice.Slice(wname="nevermatch", fallback=layout_cls())]},
+            )
+            for layout_name, layout_cls in cls.iter_layouts()
+        ]
 
 
 class AllLayouts(AllLayoutsConfig):
@@ -107,14 +112,15 @@ class AllLayouts(AllLayoutsConfig):
     Like AllLayoutsConfig, but all the layouts in the repo are installed
     together in the 'layouts' variable.
     """
-    layouts = [layout_cls() for layout_name, layout_cls
-               in AllLayoutsConfig.iter_layouts()]
+
+    layouts = [layout_cls() for layout_name, layout_cls in AllLayoutsConfig.iter_layouts()]
 
 
 class AllLayoutsConfigEvents(AllLayoutsConfig):
     """
     Extends AllLayoutsConfig to test events.
     """
+
     def __init__(self, *args, **kwargs):
         super().__init__(self, *args, **kwargs)
         # TODO: Test more events
@@ -122,18 +128,47 @@ class AllLayoutsConfigEvents(AllLayoutsConfig):
         @libqtile.hook.subscribe.startup
         def _():
             libqtile.qtile.test_data = {
-                'focus_change': 0,
+                "focus_change": 0,
             }
 
         @libqtile.hook.subscribe.focus_change
         def _():
-            libqtile.qtile.test_data['focus_change'] += 1
+            libqtile.qtile.test_data["focus_change"] += 1
 
 
-each_layout_config = pytest.mark.parametrize("manager", AllLayoutsConfig.generate(), indirect=True)
+each_layout_config = pytest.mark.parametrize(
+    "manager", AllLayoutsConfig.generate(), indirect=True
+)
 all_layouts_config = pytest.mark.parametrize("manager", [AllLayouts], indirect=True)
-each_layout_config_events = pytest.mark.parametrize("manager", AllLayoutsConfigEvents.generate(), indirect=True)
-each_delegate_layout_config = pytest.mark.parametrize("manager", AllDelegateLayoutsConfig.generate(), indirect=True)
+each_layout_config_events = pytest.mark.parametrize(
+    "manager", AllLayoutsConfigEvents.generate(), indirect=True
+)
+each_delegate_layout_config = pytest.mark.parametrize(
+    "manager", AllDelegateLayoutsConfig.generate(), indirect=True
+)
+
+
+@each_layout_config
+def test_window_order_fullscreen(manager):
+    # Add a window to fullscreen
+    manager.test_window("tofullscreen")
+
+    # windows to add after the fullscreen window
+    windows_after = ["two", "three"]
+
+    # Add some windows before
+    for win in windows_after:
+        manager.test_window(win)
+
+    windows_order = manager.c.layout.info()["clients"]
+
+    # Focus window and toggle fullscreen
+    manager.c.group.focus_by_name("tofullscreen")
+    manager.c.window.toggle_fullscreen()
+    manager.c.window.toggle_fullscreen()
+
+    # Windows must be sorted in the same order as they were created
+    assert windows_order == manager.c.layout.info()["clients"]
 
 
 @each_layout_config
@@ -148,7 +183,7 @@ def test_window_types(manager):
     manager.test_window("dialog", floating=True)
     assert_focused(manager, "dialog")
     manager.test_notification("notification")
-    assert manager.c.group.info()['focus'] == 'dialog'
+    assert manager.c.group.info()["focus"] == "dialog"
     for window in manager.c.windows():
         if window["name"] in ("dialog", "notification"):
             assert window["floating"]
@@ -163,11 +198,11 @@ def test_focus_cycle(manager):
     manager.test_window("three")
 
     # Test preconditions (the order of items in 'clients' is managed by each layout)
-    assert set(manager.c.layout.info()['clients']) == {'one', 'two', 'three'}
+    assert set(manager.c.layout.info()["clients"]) == {"one", "two", "three"}
     assert_focused(manager, "three")
 
     # Assert that the layout cycles the focus on all windows
-    assert_focus_path_unordered(manager, 'float1', 'float2', 'one', 'two', 'three')
+    assert_focus_path_unordered(manager, "float1", "float2", "one", "two", "three")
 
 
 @each_layout_config
@@ -216,55 +251,55 @@ def test_focus_change_event(manager):
     # In short, this test prevents layouts from influencing each other in
     # unexpected ways.
 
-    assert manager.c.get_test_data()['focus_change'] == 0
+    assert manager.c.get_test_data()["focus_change"] == 0
 
     # Spawning a window must fire only 1 focus_change event
     one = manager.test_window("one")
-    assert manager.c.get_test_data()['focus_change'] == 1
+    assert manager.c.get_test_data()["focus_change"] == 1
     two = manager.test_window("two")
-    assert manager.c.get_test_data()['focus_change'] == 2
+    assert manager.c.get_test_data()["focus_change"] == 2
     three = manager.test_window("three")
-    assert manager.c.get_test_data()['focus_change'] == 3
+    assert manager.c.get_test_data()["focus_change"] == 3
 
     # Switching window must fire only 1 focus_change event
     assert_focused(manager, "three")
     manager.c.group.focus_by_name("one")
-    assert manager.c.get_test_data()['focus_change'] == 4
+    assert manager.c.get_test_data()["focus_change"] == 4
     assert_focused(manager, "one")
 
     # Focusing the current window must fire another focus_change event
     manager.c.group.focus_by_name("one")
-    assert manager.c.get_test_data()['focus_change'] == 5
+    assert manager.c.get_test_data()["focus_change"] == 5
 
     # Toggling a window floating should not fire focus_change events
     manager.c.window.toggle_floating()
-    assert manager.c.get_test_data()['focus_change'] == 5
+    assert manager.c.get_test_data()["focus_change"] == 5
     manager.c.window.toggle_floating()
-    assert manager.c.get_test_data()['focus_change'] == 5
+    assert manager.c.get_test_data()["focus_change"] == 5
 
     # Removing the focused window must fire only 1 focus_change event
     assert_focused(manager, "one")
-    assert manager.c.group.info()['focus_history'] == ["two", "three", "one"]
+    assert manager.c.group.info()["focus_history"] == ["two", "three", "one"]
     manager.kill_window(one)
-    assert manager.c.get_test_data()['focus_change'] == 6
+    assert manager.c.get_test_data()["focus_change"] == 6
 
     # The position where 'one' was after it was floated and unfloated
     # above depends on the layout, so we can't predict here what window gets
     # selected after killing it; for this reason, focus 'three' explicitly to
     # continue testing
     manager.c.group.focus_by_name("three")
-    assert manager.c.group.info()['focus_history'] == ["two", "three"]
-    assert manager.c.get_test_data()['focus_change'] == 7
+    assert manager.c.group.info()["focus_history"] == ["two", "three"]
+    assert manager.c.get_test_data()["focus_change"] == 7
 
     # Removing a non-focused window must not fire focus_change events
     manager.kill_window(two)
-    assert manager.c.get_test_data()['focus_change'] == 7
+    assert manager.c.get_test_data()["focus_change"] == 7
     assert_focused(manager, "three")
 
     # Removing the last window must still generate 1 focus_change event
     manager.kill_window(three)
-    assert manager.c.layout.info()['clients'] == []
-    assert manager.c.get_test_data()['focus_change'] == 8
+    assert manager.c.layout.info()["clients"] == []
+    assert manager.c.get_test_data()["focus_change"] == 8
 
 
 @each_layout_config
@@ -273,22 +308,22 @@ def test_remove(manager):
     two = manager.test_window("two")
     three = manager.test_window("three")
     assert_focused(manager, "three")
-    assert manager.c.group.info()['focus_history'] == ["one", "two", "three"]
+    assert manager.c.group.info()["focus_history"] == ["one", "two", "three"]
 
     # Removing a focused window must focus another (which one depends on the layout)
     manager.kill_window(three)
-    assert manager.c.window.info()['name'] in manager.c.layout.info()['clients']
+    assert manager.c.window.info()["name"] in manager.c.layout.info()["clients"]
 
     # To continue testing, explicitly set focus on 'two'
     manager.c.group.focus_by_name("two")
     manager.test_window("four")
     assert_focused(manager, "four")
-    assert manager.c.group.info()['focus_history'] == ["one", "two", "four"]
+    assert manager.c.group.info()["focus_history"] == ["one", "two", "four"]
 
     # Removing a non-focused window must not change the current focus
     manager.kill_window(two)
     assert_focused(manager, "four")
-    assert manager.c.group.info()['focus_history'] == ["one", "four"]
+    assert manager.c.group.info()["focus_history"] == ["one", "four"]
 
     # Add more windows and shuffle the focus order
     five = manager.test_window("five")
@@ -297,17 +332,16 @@ def test_remove(manager):
     seven = manager.test_window("seven")
     manager.c.group.focus_by_name("six")
     assert_focused(manager, "six")
-    assert manager.c.group.info()['focus_history'] == ["four", "five", "one",
-                                                       "seven", "six"]
+    assert manager.c.group.info()["focus_history"] == ["four", "five", "one", "seven", "six"]
 
     manager.kill_window(five)
     manager.kill_window(one)
     assert_focused(manager, "six")
-    assert manager.c.group.info()['focus_history'] == ["four", "seven", "six"]
+    assert manager.c.group.info()["focus_history"] == ["four", "seven", "six"]
 
     manager.c.group.focus_by_name("seven")
     manager.kill_window(seven)
-    assert manager.c.window.info()['name'] in manager.c.layout.info()['clients']
+    assert manager.c.window.info()["name"] in manager.c.layout.info()["clients"]
 
 
 @each_layout_config
@@ -316,23 +350,23 @@ def test_remove_floating(manager):
     manager.test_window("two")
     float1 = manager.test_window("float1", floating=True)
     assert_focused(manager, "float1")
-    assert set(manager.c.layout.info()['clients']) == {"one", "two"}
-    assert manager.c.group.info()['focus_history'] == ["one", "two", "float1"]
+    assert set(manager.c.layout.info()["clients"]) == {"one", "two"}
+    assert manager.c.group.info()["focus_history"] == ["one", "two", "float1"]
 
     # Removing a focused floating window must focus the one that was focused before
     manager.kill_window(float1)
     assert_focused(manager, "two")
-    assert manager.c.group.info()['focus_history'] == ["one", "two"]
+    assert manager.c.group.info()["focus_history"] == ["one", "two"]
 
     float2 = manager.test_window("float2", floating=True)
     assert_focused(manager, "float2")
-    assert manager.c.group.info()['focus_history'] == ["one", "two", "float2"]
+    assert manager.c.group.info()["focus_history"] == ["one", "two", "float2"]
 
     # Removing a non-focused floating window must not change the current focus
     manager.c.group.focus_by_name("two")
     manager.kill_window(float2)
     assert_focused(manager, "two")
-    assert manager.c.group.info()['focus_history'] == ["one", "two"]
+    assert manager.c.group.info()["focus_history"] == ["one", "two"]
 
     # Add more windows and shuffle the focus order
     manager.test_window("three")
@@ -342,31 +376,48 @@ def test_remove_floating(manager):
     float5 = manager.test_window("float5", floating=True)
     manager.c.group.focus_by_name("three")
     manager.c.group.focus_by_name("float3")
-    assert manager.c.group.info()['focus_history'] == ["two", "one", "float4",
-                                                       "float5", "three", "float3"]
+    assert manager.c.group.info()["focus_history"] == [
+        "two",
+        "one",
+        "float4",
+        "float5",
+        "three",
+        "float3",
+    ]
 
     manager.kill_window(one)
     assert_focused(manager, "float3")
-    assert manager.c.group.info()['focus_history'] == ["two", "float4",
-                                                       "float5", "three", "float3"]
+    assert manager.c.group.info()["focus_history"] == [
+        "two",
+        "float4",
+        "float5",
+        "three",
+        "float3",
+    ]
 
     manager.kill_window(float5)
     assert_focused(manager, "float3")
-    assert manager.c.group.info()['focus_history'] == ["two", "float4", "three", "float3"]
+    assert manager.c.group.info()["focus_history"] == ["two", "float4", "three", "float3"]
 
     # The focus must be given to the previous window even if it's floating
     manager.c.group.focus_by_name("float4")
-    assert manager.c.group.info()['focus_history'] == ["two", "three", "float3", "float4"]
+    assert manager.c.group.info()["focus_history"] == ["two", "three", "float3", "float4"]
     manager.kill_window(float4)
     assert_focused(manager, "float3")
-    assert manager.c.group.info()['focus_history'] == ["two", "three", "float3"]
+    assert manager.c.group.info()["focus_history"] == ["two", "three", "float3"]
 
     four = manager.test_window("four")
     float6 = manager.test_window("float6", floating=True)
     five = manager.test_window("five")
     manager.c.group.focus_by_name("float3")
-    assert manager.c.group.info()['focus_history'] == ["two", "three", "four",
-                                                       "float6", "five", "float3"]
+    assert manager.c.group.info()["focus_history"] == [
+        "two",
+        "three",
+        "four",
+        "float6",
+        "five",
+        "float3",
+    ]
 
     # Killing several unfocused windows before the current one, and then
     # killing the current window, must focus the remaining most recently
@@ -374,10 +425,10 @@ def test_remove_floating(manager):
     manager.kill_window(five)
     manager.kill_window(four)
     manager.kill_window(float6)
-    assert manager.c.group.info()['focus_history'] == ["two", "three", "float3"]
+    assert manager.c.group.info()["focus_history"] == ["two", "three", "float3"]
     manager.kill_window(float3)
     assert_focused(manager, "three")
-    assert manager.c.group.info()['focus_history'] == ["two", "three"]
+    assert manager.c.group.info()["focus_history"] == ["two", "three"]
 
 
 @each_layout_config
@@ -389,18 +440,18 @@ def test_desktop_notifications(manager):
 
     # A notification fired in an empty group must not take focus
     notif1 = manager.test_notification("notif1")
-    assert manager.c.group.info()['focus'] is None
+    assert manager.c.group.info()["focus"] is None
     manager.kill_window(notif1)
 
     # A window is spawned while a notification is displayed
     notif2 = manager.test_notification("notif2")
     one = manager.test_window("one")
-    assert manager.c.group.info()['focus_history'] == ["one"]
+    assert manager.c.group.info()["focus_history"] == ["one"]
     manager.kill_window(notif2)
 
     # Another notification is fired, but the focus must not change
     notif3 = manager.test_notification("notif3")
-    assert_focused(manager, 'one')
+    assert_focused(manager, "one")
     manager.kill_window(notif3)
 
     # Complicate the scenario with multiple windows and notifications
@@ -409,7 +460,7 @@ def test_desktop_notifications(manager):
     manager.test_window("two")
     notif4 = manager.test_notification("notif4")
     notif5 = manager.test_notification("notif5")
-    assert manager.c.group.info()['focus_history'] == ["one", "dialog1", "two"]
+    assert manager.c.group.info()["focus_history"] == ["one", "dialog1", "two"]
 
     dialog2 = manager.test_window("dialog2", floating=True)
     manager.kill_window(notif5)
@@ -420,8 +471,7 @@ def test_desktop_notifications(manager):
     notif7 = manager.test_notification("notif7")
     manager.kill_window(notif4)
     notif8 = manager.test_notification("notif8")
-    assert manager.c.group.info()['focus_history'] == ["dialog1", "dialog2",
-                                                       "three", "two"]
+    assert manager.c.group.info()["focus_history"] == ["dialog1", "dialog2", "three", "two"]
 
     manager.test_window("dialog3", floating=True)
     manager.kill_window(dialog1)
@@ -430,7 +480,7 @@ def test_desktop_notifications(manager):
     manager.c.group.focus_by_name("three")
     manager.kill_window(notif7)
     manager.kill_window(notif8)
-    assert manager.c.group.info()['focus_history'] == ["two", "dialog3", "three"]
+    assert manager.c.group.info()["focus_history"] == ["two", "dialog3", "three"]
 
 
 @each_delegate_layout_config
@@ -438,7 +488,7 @@ def test_only_uses_delegated_screen_rect(manager):
     manager.test_window("one")
     manager.c.group.focus_by_name("one")
     assert_focused(manager, "one")
-    assert_dimensions_fit(manager, 256, 0, 800-256, 600)
+    assert_dimensions_fit(manager, 256, 0, 800 - 256, 600)
 
 
 @all_layouts_config
@@ -451,31 +501,32 @@ def test_cycle_layouts(manager):
     assert_focused(manager, "three")
 
     # Cycling all the layouts must keep the current window focused
-    initial_layout_name = manager.c.layout.info()['name']
+    initial_layout_name = manager.c.layout.info()["name"]
     while True:
         manager.c.next_layout()
-        if manager.c.layout.info()['name'] == initial_layout_name:
+        if manager.c.layout.info()["name"] == initial_layout_name:
             break
         # Use manager.c.layout.info()['name'] in the assertion message, so we
         # know which layout is buggy
-        assert manager.c.window.info()['name'] == "three", manager.c.layout.info()['name']
+        assert manager.c.window.info()["name"] == "three", manager.c.layout.info()["name"]
 
     # Now try backwards
     while True:
         manager.c.prev_layout()
-        if manager.c.layout.info()['name'] == initial_layout_name:
+        if manager.c.layout.info()["name"] == initial_layout_name:
             break
         # Use manager.c.layout.info()['name'] in the assertion message, so we
         # know which layout is buggy
-        assert manager.c.window.info()['name'] == "three", manager.c.layout.info()['name']
+        assert manager.c.window.info()["name"] == "three", manager.c.layout.info()["name"]
 
 
 class AllLayoutsMultipleBorders(AllLayoutsConfig):
     """
     Like AllLayouts, but all the layouts have border_focus set to a list of colors.
     """
+
     layouts = [
-        layout_cls(border_focus=['#000' '#111', '#222', '#333', '#444'])
+        layout_cls(border_focus=["#000" "#111", "#222", "#333", "#444"])
         for layout_name, layout_cls in AllLayoutsConfig.iter_layouts()
     ]
 
@@ -485,8 +536,8 @@ def test_multiple_borders(manager):
     manager.test_window("one")
     manager.test_window("two")
 
-    initial_layout_name = manager.c.layout.info()['name']
+    initial_layout_name = manager.c.layout.info()["name"]
     while True:
         manager.c.next_layout()
-        if manager.c.layout.info()['name'] == initial_layout_name:
+        if manager.c.layout.info()["name"] == initial_layout_name:
             break
