@@ -23,21 +23,13 @@ import pytest
 import libqtile.config
 from libqtile import layout
 from libqtile.confreader import Config
-from test.layouts.layout_utils import (
-    assert_dimensions,
-    assert_focus_path,
-    assert_focused,
-)
+from test.layouts.layout_utils import assert_dimensions, assert_focus_path, assert_focused
 
 
 class MonadTallConfig(Config):
     auto_fullscreen = True
-    groups = [
-        libqtile.config.Group("a")
-    ]
-    layouts = [
-        layout.MonadTall()
-    ]
+    groups = [libqtile.config.Group("a")]
+    layouts = [layout.MonadTall()]
     floating_layout = libqtile.resources.default_config.floating_layout
     keys = []
     mouse = []
@@ -50,12 +42,8 @@ monadtall_config = pytest.mark.parametrize("manager", [MonadTallConfig], indirec
 
 class MonadTallNCPBeforeCurrentConfig(Config):
     auto_fullscreen = True
-    groups = [
-        libqtile.config.Group("a")
-    ]
-    layouts = [
-        layout.MonadTall(new_client_position='before_current')
-    ]
+    groups = [libqtile.config.Group("a")]
+    layouts = [layout.MonadTall(new_client_position="before_current")]
     floating_layout = libqtile.resources.default_config.floating_layout
     keys = []
     mouse = []
@@ -63,17 +51,15 @@ class MonadTallNCPBeforeCurrentConfig(Config):
     follow_mouse_focus = False
 
 
-monadtallncpbeforecurrent_config = pytest.mark.parametrize("manager", [MonadTallNCPBeforeCurrentConfig], indirect=True)
+monadtallncpbeforecurrent_config = pytest.mark.parametrize(
+    "manager", [MonadTallNCPBeforeCurrentConfig], indirect=True
+)
 
 
 class MonadTallNCPAfterCurrentConfig(Config):
     auto_fullscreen = True
-    groups = [
-        libqtile.config.Group("a")
-    ]
-    layouts = [
-        layout.MonadTall(new_client_position="after_current")
-    ]
+    groups = [libqtile.config.Group("a")]
+    layouts = [layout.MonadTall(new_client_position="after_current")]
     floating_layout = libqtile.resources.default_config.floating_layout
     keys = []
     mouse = []
@@ -81,17 +67,15 @@ class MonadTallNCPAfterCurrentConfig(Config):
     follow_mouse_focus = False
 
 
-monadtallncpaftercurrent_config = pytest.mark.parametrize("manager", [MonadTallNCPAfterCurrentConfig], indirect=True)
+monadtallncpaftercurrent_config = pytest.mark.parametrize(
+    "manager", [MonadTallNCPAfterCurrentConfig], indirect=True
+)
 
 
 class MonadTallNewCLientPositionBottomConfig(Config):
     auto_fullscreen = True
-    groups = [
-        libqtile.config.Group("a")
-    ]
-    layouts = [
-        layout.MonadTall(new_client_position="bottom")
-    ]
+    groups = [libqtile.config.Group("a")]
+    layouts = [layout.MonadTall(new_client_position="bottom")]
     floating_layout = libqtile.resources.default_config.floating_layout
     keys = []
     mouse = []
@@ -101,12 +85,8 @@ class MonadTallNewCLientPositionBottomConfig(Config):
 
 class MonadTallMarginsConfig(Config):
     auto_fullscreen = True
-    groups = [
-        libqtile.config.Group("a")
-    ]
-    layouts = [
-        layout.MonadTall(margin=4)
-    ]
+    groups = [libqtile.config.Group("a")]
+    layouts = [layout.MonadTall(margin=4)]
     floating_layout = libqtile.resources.default_config.floating_layout
     keys = []
     mouse = []
@@ -114,17 +94,15 @@ class MonadTallMarginsConfig(Config):
     follow_mouse_focus = False
 
 
-monadtallmargins_config = pytest.mark.parametrize("manager", [MonadTallMarginsConfig], indirect=True)
+monadtallmargins_config = pytest.mark.parametrize(
+    "manager", [MonadTallMarginsConfig], indirect=True
+)
 
 
 class MonadWideConfig(Config):
     auto_fullscreen = True
-    groups = [
-        libqtile.config.Group("a")
-    ]
-    layouts = [
-        layout.MonadWide()
-    ]
+    groups = [libqtile.config.Group("a")]
+    layouts = [layout.MonadWide()]
     floating_layout = libqtile.resources.default_config.floating_layout
     keys = []
     mouse = []
@@ -137,12 +115,8 @@ monadwide_config = pytest.mark.parametrize("manager", [MonadWideConfig], indirec
 
 class MonadWideNewClientPositionTopConfig(Config):
     auto_fullscreen = True
-    groups = [
-        libqtile.config.Group("a")
-    ]
-    layouts = [
-        layout.MonadWide(new_client_position="top")
-    ]
+    groups = [libqtile.config.Group("a")]
+    layouts = [layout.MonadWide(new_client_position="top")]
     floating_layout = libqtile.resources.default_config.floating_layout
     keys = []
     mouse = []
@@ -152,12 +126,8 @@ class MonadWideNewClientPositionTopConfig(Config):
 
 class MonadWideMarginsConfig(Config):
     auto_fullscreen = True
-    groups = [
-        libqtile.config.Group("a")
-    ]
-    layouts = [
-        layout.MonadWide(margin=4)
-    ]
+    groups = [libqtile.config.Group("a")]
+    layouts = [layout.MonadWide(margin=4)]
     floating_layout = libqtile.resources.default_config.floating_layout
     keys = []
     mouse = []
@@ -167,147 +137,147 @@ class MonadWideMarginsConfig(Config):
 
 @monadtall_config
 def test_tall_add_clients(manager):
-    manager.test_window('one')
-    manager.test_window('two')
-    assert manager.c.layout.info()["main"] == 'one'
-    assert manager.c.layout.info()["secondary"] == ['two']
-    assert_focused(manager, 'two')
+    manager.test_window("one")
+    manager.test_window("two")
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two"]
+    assert_focused(manager, "two")
 
-    manager.test_window('three')
-    assert manager.c.layout.info()["main"] == 'one'
-    assert manager.c.layout.info()["secondary"] == ['two', 'three']
-    assert_focused(manager, 'three')
+    manager.test_window("three")
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "three"]
+    assert_focused(manager, "three")
 
     manager.c.layout.previous()
-    assert_focused(manager, 'two')
+    assert_focused(manager, "two")
 
-    manager.test_window('four')
-    assert manager.c.layout.info()["main"] == 'one'
-    assert manager.c.layout.info()["secondary"] == ['two', 'four', 'three']
-    assert_focused(manager, 'four')
+    manager.test_window("four")
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "four", "three"]
+    assert_focused(manager, "four")
 
 
 @monadtallncpbeforecurrent_config
 def test_tall_add_clients_before_current(manager):
-    """ Test add client with new_client_position = before_current. """
-    manager.test_window('one')
-    manager.test_window('two')
-    manager.test_window('three')
-    assert manager.c.layout.info()["main"] == 'three'
-    assert manager.c.layout.info()["secondary"] == ['two', 'one']
+    """Test add client with new_client_position = before_current."""
+    manager.test_window("one")
+    manager.test_window("two")
+    manager.test_window("three")
+    assert manager.c.layout.info()["main"] == "three"
+    assert manager.c.layout.info()["secondary"] == ["two", "one"]
     manager.c.layout.next()
-    assert_focused(manager, 'two')
-    manager.test_window('four')
-    assert manager.c.layout.info()["main"] == 'three'
-    assert manager.c.layout.info()["secondary"] == ['four', 'two', 'one']
-    assert_focused(manager, 'four')
+    assert_focused(manager, "two")
+    manager.test_window("four")
+    assert manager.c.layout.info()["main"] == "three"
+    assert manager.c.layout.info()["secondary"] == ["four", "two", "one"]
+    assert_focused(manager, "four")
 
 
 @monadtallncpaftercurrent_config
 def test_tall_add_clients_after_current(manager):
-    manager.test_window('one')
-    manager.test_window('two')
-    manager.test_window('three')
+    manager.test_window("one")
+    manager.test_window("two")
+    manager.test_window("three")
     manager.c.layout.previous()
-    assert_focused(manager, 'two')
-    manager.test_window('four')
-    assert manager.c.layout.info()["main"] == 'one'
-    assert manager.c.layout.info()["secondary"] == ['two', 'four', 'three']
-    assert_focused(manager, 'four')
+    assert_focused(manager, "two")
+    manager.test_window("four")
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "four", "three"]
+    assert_focused(manager, "four")
 
 
 @pytest.mark.parametrize("manager", [MonadTallNewCLientPositionBottomConfig], indirect=True)
 def test_tall_add_clients_at_bottom(manager):
-    manager.test_window('one')
-    manager.test_window('two')
-    manager.test_window('three')
+    manager.test_window("one")
+    manager.test_window("two")
+    manager.test_window("three")
     manager.c.layout.previous()
-    assert_focused(manager, 'two')
-    manager.test_window('four')
-    assert manager.c.layout.info()["main"] == 'one'
-    assert manager.c.layout.info()["secondary"] == ['two', 'three', 'four']
+    assert_focused(manager, "two")
+    manager.test_window("four")
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "three", "four"]
 
 
 @monadwide_config
 def test_wide_add_clients(manager):
-    manager.test_window('one')
-    manager.test_window('two')
-    assert manager.c.layout.info()["main"] == 'one'
-    assert manager.c.layout.info()["secondary"] == ['two']
-    assert_focused(manager, 'two')
+    manager.test_window("one")
+    manager.test_window("two")
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two"]
+    assert_focused(manager, "two")
 
-    manager.test_window('three')
-    assert manager.c.layout.info()["main"] == 'one'
-    assert manager.c.layout.info()["secondary"] == ['two', 'three']
-    assert_focused(manager, 'three')
+    manager.test_window("three")
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "three"]
+    assert_focused(manager, "three")
 
     manager.c.layout.previous()
-    assert_focused(manager, 'two')
+    assert_focused(manager, "two")
 
-    manager.test_window('four')
-    assert manager.c.layout.info()["main"] == 'one'
-    assert manager.c.layout.info()["secondary"] == ['two', 'four', 'three']
-    assert_focused(manager, 'four')
+    manager.test_window("four")
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "four", "three"]
+    assert_focused(manager, "four")
 
 
 @pytest.mark.parametrize("manager", [MonadWideNewClientPositionTopConfig], indirect=True)
 def test_wide_add_clients_new_client_postion_top(manager):
-    manager.test_window('one')
-    manager.test_window('two')
-    assert manager.c.layout.info()["main"] == 'two'
-    assert manager.c.layout.info()["secondary"] == ['one']
-    assert_focused(manager, 'two')
+    manager.test_window("one")
+    manager.test_window("two")
+    assert manager.c.layout.info()["main"] == "two"
+    assert manager.c.layout.info()["secondary"] == ["one"]
+    assert_focused(manager, "two")
 
-    manager.test_window('three')
-    assert manager.c.layout.info()["main"] == 'three'
-    assert manager.c.layout.info()["secondary"] == ['two', 'one']
-    assert_focused(manager, 'three')
+    manager.test_window("three")
+    assert manager.c.layout.info()["main"] == "three"
+    assert manager.c.layout.info()["secondary"] == ["two", "one"]
+    assert_focused(manager, "three")
 
     manager.c.layout.next()
-    assert_focused(manager, 'two')
+    assert_focused(manager, "two")
 
-    manager.test_window('four')
-    assert manager.c.layout.info()["main"] == 'four'
-    assert manager.c.layout.info()["secondary"] == ['three', 'two', 'one']
-    assert_focused(manager, 'four')
+    manager.test_window("four")
+    assert manager.c.layout.info()["main"] == "four"
+    assert manager.c.layout.info()["secondary"] == ["three", "two", "one"]
+    assert_focused(manager, "four")
 
 
 @monadtallmargins_config
 def test_tall_margins(manager):
-    manager.test_window('one')
+    manager.test_window("one")
     assert_dimensions(manager, 4, 4, 788, 588)
 
-    manager.test_window('two')
-    assert_focused(manager, 'two')
+    manager.test_window("two")
+    assert_focused(manager, "two")
     assert_dimensions(manager, 404, 4, 388, 588)
 
     manager.c.layout.previous()
-    assert_focused(manager, 'one')
+    assert_focused(manager, "one")
     assert_dimensions(manager, 4, 4, 392, 588)
 
 
 @pytest.mark.parametrize("manager", [MonadWideMarginsConfig], indirect=True)
 def test_wide_margins(manager):
-    manager.test_window('one')
+    manager.test_window("one")
     assert_dimensions(manager, 4, 4, 788, 588)
 
-    manager.test_window('two')
-    assert_focused(manager, 'two')
+    manager.test_window("two")
+    assert_focused(manager, "two")
     assert_dimensions(manager, 4, 304, 788, 288)
 
     manager.c.layout.previous()
-    assert_focused(manager, 'one')
+    assert_focused(manager, "one")
     assert_dimensions(manager, 4, 4, 788, 292)
 
 
 @monadtall_config
 def test_tall_growmain_solosecondary(manager):
-    manager.test_window('one')
+    manager.test_window("one")
     assert_dimensions(manager, 0, 0, 796, 596)
 
-    manager.test_window('two')
+    manager.test_window("two")
     manager.c.layout.previous()
-    assert_focused(manager, 'one')
+    assert_focused(manager, "one")
 
     assert_dimensions(manager, 0, 0, 396, 596)
     manager.c.layout.grow()
@@ -329,12 +299,12 @@ def test_tall_growmain_solosecondary(manager):
 
 @monadwide_config
 def test_wide_growmain_solosecondary(manager):
-    manager.test_window('one')
+    manager.test_window("one")
     assert_dimensions(manager, 0, 0, 796, 596)
 
-    manager.test_window('two')
+    manager.test_window("two")
     manager.c.layout.previous()
-    assert_focused(manager, 'one')
+    assert_focused(manager, "one")
 
     assert_dimensions(manager, 0, 0, 796, 296)
     manager.c.layout.grow()
@@ -356,14 +326,14 @@ def test_wide_growmain_solosecondary(manager):
 
 @monadtall_config
 def test_tall_growmain_multiplesecondary(manager):
-    manager.test_window('one')
+    manager.test_window("one")
     assert_dimensions(manager, 0, 0, 796, 596)
 
-    manager.test_window('two')
-    manager.test_window('three')
+    manager.test_window("two")
+    manager.test_window("three")
     manager.c.layout.previous()
     manager.c.layout.previous()
-    assert_focused(manager, 'one')
+    assert_focused(manager, "one")
 
     assert_dimensions(manager, 0, 0, 396, 596)
     manager.c.layout.grow()
@@ -385,14 +355,14 @@ def test_tall_growmain_multiplesecondary(manager):
 
 @monadwide_config
 def test_wide_growmain_multiplesecondary(manager):
-    manager.test_window('one')
+    manager.test_window("one")
     assert_dimensions(manager, 0, 0, 796, 596)
 
-    manager.test_window('two')
-    manager.test_window('three')
+    manager.test_window("two")
+    manager.test_window("three")
     manager.c.layout.previous()
     manager.c.layout.previous()
-    assert_focused(manager, 'one')
+    assert_focused(manager, "one")
 
     assert_dimensions(manager, 0, 0, 796, 296)
     manager.c.layout.grow()
@@ -414,11 +384,11 @@ def test_wide_growmain_multiplesecondary(manager):
 
 @monadtall_config
 def test_tall_growsecondary_solosecondary(manager):
-    manager.test_window('one')
+    manager.test_window("one")
     assert_dimensions(manager, 0, 0, 796, 596)
 
-    manager.test_window('two')
-    assert_focused(manager, 'two')
+    manager.test_window("two")
+    assert_focused(manager, "two")
 
     assert_dimensions(manager, 400, 0, 396, 596)
     manager.c.layout.grow()
@@ -440,11 +410,11 @@ def test_tall_growsecondary_solosecondary(manager):
 
 @monadwide_config
 def test_wide_growsecondary_solosecondary(manager):
-    manager.test_window('one')
+    manager.test_window("one")
     assert_dimensions(manager, 0, 0, 796, 596)
 
-    manager.test_window('two')
-    assert_focused(manager, 'two')
+    manager.test_window("two")
+    assert_focused(manager, "two")
 
     assert_dimensions(manager, 0, 300, 796, 296)
     manager.c.layout.grow()
@@ -466,13 +436,13 @@ def test_wide_growsecondary_solosecondary(manager):
 
 @monadtall_config
 def test_tall_growsecondary_multiplesecondary(manager):
-    manager.test_window('one')
+    manager.test_window("one")
     assert_dimensions(manager, 0, 0, 796, 596)
 
-    manager.test_window('two')
-    manager.test_window('three')
+    manager.test_window("two")
+    manager.test_window("three")
     manager.c.layout.previous()
-    assert_focused(manager, 'two')
+    assert_focused(manager, "two")
 
     assert_dimensions(manager, 400, 0, 396, 296)
     # Grow 20 pixels
@@ -494,13 +464,13 @@ def test_tall_growsecondary_multiplesecondary(manager):
 
 @monadwide_config
 def test_wide_growsecondary_multiplesecondary(manager):
-    manager.test_window('one')
+    manager.test_window("one")
     assert_dimensions(manager, 0, 0, 796, 596)
 
-    manager.test_window('two')
-    manager.test_window('three')
+    manager.test_window("two")
+    manager.test_window("three")
     manager.c.layout.previous()
-    assert_focused(manager, 'two')
+    assert_focused(manager, "two")
 
     assert_dimensions(manager, 0, 300, 396, 296)
     # Grow 20 pixels
@@ -522,202 +492,240 @@ def test_wide_growsecondary_multiplesecondary(manager):
 
 @monadtall_config
 def test_tall_flip(manager):
-    manager.test_window('one')
-    manager.test_window('two')
-    manager.test_window('three')
+    manager.test_window("one")
+    manager.test_window("two")
+    manager.test_window("three")
 
     # Check all the dimensions
     manager.c.layout.next()
-    assert_focused(manager, 'one')
+    assert_focused(manager, "one")
     assert_dimensions(manager, 0, 0, 396, 596)
 
     manager.c.layout.next()
-    assert_focused(manager, 'two')
+    assert_focused(manager, "two")
     assert_dimensions(manager, 400, 0, 396, 296)
 
     manager.c.layout.next()
-    assert_focused(manager, 'three')
+    assert_focused(manager, "three")
     assert_dimensions(manager, 400, 300, 396, 296)
 
     # Now flip it and do it again
     manager.c.layout.flip()
 
     manager.c.layout.next()
-    assert_focused(manager, 'one')
+    assert_focused(manager, "one")
     assert_dimensions(manager, 400, 0, 396, 596)
 
     manager.c.layout.next()
-    assert_focused(manager, 'two')
+    assert_focused(manager, "two")
     assert_dimensions(manager, 0, 0, 396, 296)
 
     manager.c.layout.next()
-    assert_focused(manager, 'three')
+    assert_focused(manager, "three")
     assert_dimensions(manager, 0, 300, 396, 296)
 
 
 @monadwide_config
 def test_wide_flip(manager):
-    manager.test_window('one')
-    manager.test_window('two')
-    manager.test_window('three')
+    manager.test_window("one")
+    manager.test_window("two")
+    manager.test_window("three")
 
     # Check all the dimensions
     manager.c.layout.next()
-    assert_focused(manager, 'one')
+    assert_focused(manager, "one")
     assert_dimensions(manager, 0, 0, 796, 296)
 
     manager.c.layout.next()
-    assert_focused(manager, 'two')
+    assert_focused(manager, "two")
     assert_dimensions(manager, 0, 300, 396, 296)
 
     manager.c.layout.next()
-    assert_focused(manager, 'three')
+    assert_focused(manager, "three")
     assert_dimensions(manager, 400, 300, 396, 296)
 
     # Now flip it and do it again
     manager.c.layout.flip()
 
     manager.c.layout.next()
-    assert_focused(manager, 'one')
+    assert_focused(manager, "one")
     assert_dimensions(manager, 0, 300, 796, 296)
 
     manager.c.layout.next()
-    assert_focused(manager, 'two')
+    assert_focused(manager, "two")
     assert_dimensions(manager, 0, 0, 396, 296)
 
     manager.c.layout.next()
-    assert_focused(manager, 'three')
+    assert_focused(manager, "three")
     assert_dimensions(manager, 400, 0, 396, 296)
 
 
 @monadtall_config
 def test_tall_shuffle(manager):
-    manager.test_window('one')
-    manager.test_window('two')
-    manager.test_window('three')
-    manager.test_window('four')
+    manager.test_window("one")
+    manager.test_window("two")
+    manager.test_window("three")
+    manager.test_window("four")
 
-    assert manager.c.layout.info()['main'] == 'one'
-    assert manager.c.layout.info()['secondary'] == ['two', 'three', 'four']
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "three", "four"]
 
     manager.c.layout.shuffle_up()
-    assert manager.c.layout.info()['main'] == 'one'
-    assert manager.c.layout.info()['secondary'] == ['two', 'four', 'three']
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "four", "three"]
 
     manager.c.layout.shuffle_up()
-    assert manager.c.layout.info()['main'] == 'one'
-    assert manager.c.layout.info()['secondary'] == ['four', 'two', 'three']
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["four", "two", "three"]
 
     manager.c.layout.shuffle_up()
-    assert manager.c.layout.info()['main'] == 'four'
-    assert manager.c.layout.info()['secondary'] == ['one', 'two', 'three']
+    assert manager.c.layout.info()["main"] == "four"
+    assert manager.c.layout.info()["secondary"] == ["one", "two", "three"]
 
 
 @monadwide_config
 def test_wide_shuffle(manager):
-    manager.test_window('one')
-    manager.test_window('two')
-    manager.test_window('three')
-    manager.test_window('four')
+    manager.test_window("one")
+    manager.test_window("two")
+    manager.test_window("three")
+    manager.test_window("four")
 
-    assert manager.c.layout.info()['main'] == 'one'
-    assert manager.c.layout.info()['secondary'] == ['two', 'three', 'four']
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "three", "four"]
 
     manager.c.layout.shuffle_up()
-    assert manager.c.layout.info()['main'] == 'one'
-    assert manager.c.layout.info()['secondary'] == ['two', 'four', 'three']
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "four", "three"]
 
     manager.c.layout.shuffle_up()
-    assert manager.c.layout.info()['main'] == 'one'
-    assert manager.c.layout.info()['secondary'] == ['four', 'two', 'three']
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["four", "two", "three"]
 
     manager.c.layout.shuffle_up()
-    assert manager.c.layout.info()['main'] == 'four'
-    assert manager.c.layout.info()['secondary'] == ['one', 'two', 'three']
+    assert manager.c.layout.info()["main"] == "four"
+    assert manager.c.layout.info()["secondary"] == ["one", "two", "three"]
 
 
 @monadtall_config
 def test_tall_swap(manager):
-    manager.test_window('one')
-    manager.test_window('two')
-    manager.test_window('three')
-    manager.test_window('focused')
+    manager.test_window("one")
+    manager.test_window("two")
+    manager.test_window("three")
+    manager.test_window("focused")
 
-    assert manager.c.layout.info()['main'] == 'one'
-    assert manager.c.layout.info()['secondary'] == ['two', 'three', 'focused']
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "three", "focused"]
 
     # Swap a secondary left, left aligned
     manager.c.layout.swap_left()
-    assert manager.c.layout.info()['main'] == 'focused'
-    assert manager.c.layout.info()['secondary'] == ['two', 'three', 'one']
+    assert manager.c.layout.info()["main"] == "focused"
+    assert manager.c.layout.info()["secondary"] == ["two", "three", "one"]
 
     # Swap a main right, left aligned
     manager.c.layout.swap_right()
-    assert manager.c.layout.info()['main'] == 'two'
-    assert manager.c.layout.info()['secondary'] == ['focused', 'three', 'one']
+    assert manager.c.layout.info()["main"] == "two"
+    assert manager.c.layout.info()["secondary"] == ["focused", "three", "one"]
 
     # flip over
     manager.c.layout.flip()
     manager.c.layout.shuffle_down()
-    assert manager.c.layout.info()['main'] == 'two'
-    assert manager.c.layout.info()['secondary'] == ['three', 'focused', 'one']
+    assert manager.c.layout.info()["main"] == "two"
+    assert manager.c.layout.info()["secondary"] == ["three", "focused", "one"]
 
     # Swap secondary right, right aligned
     manager.c.layout.swap_right()
-    assert manager.c.layout.info()['main'] == 'focused'
-    assert manager.c.layout.info()['secondary'] == ['three', 'two', 'one']
+    assert manager.c.layout.info()["main"] == "focused"
+    assert manager.c.layout.info()["secondary"] == ["three", "two", "one"]
 
     # Swap main left, right aligned
     manager.c.layout.swap_left()
-    assert manager.c.layout.info()['main'] == 'three'
-    assert manager.c.layout.info()['secondary'] == ['focused', 'two', 'one']
+    assert manager.c.layout.info()["main"] == "three"
+    assert manager.c.layout.info()["secondary"] == ["focused", "two", "one"]
 
     # Do swap main
     manager.c.layout.swap_main()
-    assert manager.c.layout.info()['main'] == 'focused'
-    assert manager.c.layout.info()['secondary'] == ['three', 'two', 'one']
+    assert manager.c.layout.info()["main"] == "focused"
+    assert manager.c.layout.info()["secondary"] == ["three", "two", "one"]
+
+    # Since the focused window is already to the right this swap shouldn't
+    # change the position of the windows
+    # The swap function will try to get all windows to the right of the
+    # focused window, which will result in a empty list that could cause
+    # an error if not handled
+
+    # Swap againts right edge
+    manager.c.layout.swap_right()
+    assert manager.c.layout.info()["main"] == "focused"
+    assert manager.c.layout.info()["secondary"] == ["three", "two", "one"]
+
+    # Same as above but for the swap_left function
+
+    # Swap againts left edge
+    manager.c.layout.swap_left()
+    manager.c.layout.swap_left()
+    assert manager.c.layout.info()["main"] == "three"
+    assert manager.c.layout.info()["secondary"] == ["focused", "two", "one"]
 
 
 @monadwide_config
 def test_wide_swap(manager):
-    manager.test_window('one')
-    manager.test_window('two')
-    manager.test_window('three')
-    manager.test_window('focused')
+    manager.test_window("one")
+    manager.test_window("two")
+    manager.test_window("three")
+    manager.test_window("focused")
 
-    assert manager.c.layout.info()['main'] == 'one'
-    assert manager.c.layout.info()['secondary'] == ['two', 'three', 'focused']
+    assert manager.c.layout.info()["main"] == "one"
+    assert manager.c.layout.info()["secondary"] == ["two", "three", "focused"]
 
     # Swap a secondary up
     manager.c.layout.swap_right()  # equivalent to swap_down
-    assert manager.c.layout.info()['main'] == 'focused'
-    assert manager.c.layout.info()['secondary'] == ['two', 'three', 'one']
+    assert manager.c.layout.info()["main"] == "focused"
+    assert manager.c.layout.info()["secondary"] == ["two", "three", "one"]
 
     # Swap a main down
     manager.c.layout.swap_left()  # equivalent to swap up
-    assert manager.c.layout.info()['main'] == 'two'
-    assert manager.c.layout.info()['secondary'] == ['focused', 'three', 'one']
+    assert manager.c.layout.info()["main"] == "two"
+    assert manager.c.layout.info()["secondary"] == ["focused", "three", "one"]
 
     # flip over
     manager.c.layout.flip()
     manager.c.layout.shuffle_down()
-    assert manager.c.layout.info()['main'] == 'two'
-    assert manager.c.layout.info()['secondary'] == ['three', 'focused', 'one']
+    assert manager.c.layout.info()["main"] == "two"
+    assert manager.c.layout.info()["secondary"] == ["three", "focused", "one"]
 
     # Swap secondary down
     manager.c.layout.swap_left()
-    assert manager.c.layout.info()['main'] == 'focused'
-    assert manager.c.layout.info()['secondary'] == ['three', 'two', 'one']
+    assert manager.c.layout.info()["main"] == "focused"
+    assert manager.c.layout.info()["secondary"] == ["three", "two", "one"]
 
     # Swap main up
     manager.c.layout.swap_right()
-    assert manager.c.layout.info()['main'] == 'three'
-    assert manager.c.layout.info()['secondary'] == ['focused', 'two', 'one']
+    assert manager.c.layout.info()["main"] == "three"
+    assert manager.c.layout.info()["secondary"] == ["focused", "two", "one"]
 
     # Do swap main
     manager.c.layout.swap_main()
-    assert manager.c.layout.info()['main'] == 'focused'
-    assert manager.c.layout.info()['secondary'] == ['three', 'two', 'one']
+    assert manager.c.layout.info()["main"] == "focused"
+    assert manager.c.layout.info()["secondary"] == ["three", "two", "one"]
+
+    # Since the focused window is already to the left this swap shouldn't
+    # change the position of the windows
+    # The swap function will try to get all windows to the left of the
+    # focused window, which will result in a empty list that could cause
+    # an error if not handled
+
+    # Swap againts left edge
+    manager.c.layout.swap_left()
+    assert manager.c.layout.info()["main"] == "focused"
+    assert manager.c.layout.info()["secondary"] == ["three", "two", "one"]
+
+    # Same as above but for the swap_right function
+
+    # Swap againts right edge
+    manager.c.layout.swap_right()
+    manager.c.layout.swap_right()
+    assert manager.c.layout.info()["main"] == "three"
+    assert manager.c.layout.info()["secondary"] == ["focused", "two", "one"]
 
 
 @monadtall_config
@@ -732,13 +740,13 @@ def test_tall_window_focus_cycle(manager):
     manager.test_window("three")
 
     # test preconditions
-    assert manager.c.layout.info()['clients'] == ['one', 'two', 'three']
+    assert manager.c.layout.info()["clients"] == ["one", "two", "three"]
     # last added window has focus
     assert_focused(manager, "three")
 
     # starting from the last tiled client, we first cycle through floating ones,
     # and afterwards through the tiled
-    assert_focus_path(manager, 'float1', 'float2', 'one', 'two', 'three')
+    assert_focus_path(manager, "float1", "float2", "one", "two", "three")
 
 
 @monadwide_config
@@ -753,9 +761,9 @@ def test_wide_window_focus_cycle(manager):
     manager.test_window("three")
 
     # test preconditions
-    assert manager.c.layout.info()['clients'] == ['one', 'two', 'three']
+    assert manager.c.layout.info()["clients"] == ["one", "two", "three"]
     # last added window has focus
     assert_focused(manager, "three")
 
     # assert window focus cycle, according to order in layout
-    assert_focus_path(manager, 'float1', 'float2', 'one', 'two', 'three')
+    assert_focus_path(manager, "float1", "float2", "one", "two", "three")
