@@ -6,46 +6,64 @@ from test.test_manager import ManagerConfig
 bare_config = pytest.mark.parametrize("manager", [BareConfig], indirect=True)
 
 
+@bare_config
+def test_info(manager):
+    """
+    Checks each backend Window implementation provides the required information.
+    """
+    manager.test_window("one")
+    manager.c.sync()
+    info = manager.c.window.info()
+    assert info["name"] == "one"
+    assert info["group"] == "a"
+    assert info["wm_class"][0] == "TestWindow"
+    assert "x" in info
+    assert "y" in info
+    assert "width" in info
+    assert "height" in info
+    assert "id" in info
+
+
 @bare_config
 def test_margin(manager):
-    manager.test_window('one')
+    manager.test_window("one")
 
     # No margin
-    manager.c.window.place(10, 20, 50, 60, 0, '000000')
-    assert manager.c.window.info()['x'] == 10
-    assert manager.c.window.info()['y'] == 20
-    assert manager.c.window.info()['width'] == 50
-    assert manager.c.window.info()['height'] == 60
+    manager.c.window.place(10, 20, 50, 60, 0, "000000")
+    assert manager.c.window.info()["x"] == 10
+    assert manager.c.window.info()["y"] == 20
+    assert manager.c.window.info()["width"] == 50
+    assert manager.c.window.info()["height"] == 60
 
     # Margin as int
-    manager.c.window.place(10, 20, 50, 60, 0, '000000', margin=8)
-    assert manager.c.window.info()['x'] == 18
-    assert manager.c.window.info()['y'] == 28
-    assert manager.c.window.info()['width'] == 34
-    assert manager.c.window.info()['height'] == 44
+    manager.c.window.place(10, 20, 50, 60, 0, "000000", margin=8)
+    assert manager.c.window.info()["x"] == 18
+    assert manager.c.window.info()["y"] == 28
+    assert manager.c.window.info()["width"] == 34
+    assert manager.c.window.info()["height"] == 44
 
     # Margin as list
-    manager.c.window.place(10, 20, 50, 60, 0, '000000', margin=[2, 4, 8, 10])
-    assert manager.c.window.info()['x'] == 20
-    assert manager.c.window.info()['y'] == 22
-    assert manager.c.window.info()['width'] == 36
-    assert manager.c.window.info()['height'] == 50
+    manager.c.window.place(10, 20, 50, 60, 0, "000000", margin=[2, 4, 8, 10])
+    assert manager.c.window.info()["x"] == 20
+    assert manager.c.window.info()["y"] == 22
+    assert manager.c.window.info()["width"] == 36
+    assert manager.c.window.info()["height"] == 50
 
 
 @bare_config
 def test_no_size_hint(manager):
-    manager.test_window('one')
+    manager.test_window("one")
     manager.c.window.enable_floating()
-    assert manager.c.window.info()['width'] == 100
-    assert manager.c.window.info()['height'] == 100
+    assert manager.c.window.info()["width"] == 100
+    assert manager.c.window.info()["height"] == 100
 
     manager.c.window.set_size_floating(50, 50)
-    assert manager.c.window.info()['width'] == 50
-    assert manager.c.window.info()['height'] == 50
+    assert manager.c.window.info()["width"] == 50
+    assert manager.c.window.info()["height"] == 50
 
     manager.c.window.set_size_floating(200, 200)
-    assert manager.c.window.info()['width'] == 200
-    assert manager.c.window.info()['height'] == 200
+    assert manager.c.window.info()["width"] == 200
+    assert manager.c.window.info()["height"] == 200
 
 
 class BringFrontClickConfig(ManagerConfig):
