@@ -1,10 +1,6 @@
 import libqtile.config
-from libqtile.bar import Bar
 from libqtile.widget import TextBox, WidgetBox
-
-
-def no_op(*args, **kwargs):
-    pass
+from test.widgets.conftest import FakeBar
 
 
 def test_widgetbox_widget(fake_qtile, fake_window):
@@ -13,16 +9,10 @@ def test_widgetbox_widget(fake_qtile, fake_window):
     tb_two = TextBox(name="tb_two", text="TB TWO")
 
     # Give widgetbox invalid value for button location
-    widget_box = WidgetBox([tb_one, tb_two],
-                           close_button_location="middle",
-                           fontsize=10)
+    widget_box = WidgetBox([tb_one, tb_two], close_button_location="middle", fontsize=10)
 
     # Create a bar and set attributes needed to run widget
-    fakebar = Bar([widget_box], 24)
-    fakebar.window = fake_window
-    fakebar.width = 10
-    fakebar.height = 10
-    fakebar.draw = no_op
+    fakebar = FakeBar([widget_box], window=fake_window)
 
     # Configure the widget box
     widget_box._configure(fake_qtile, fakebar)
@@ -64,11 +54,7 @@ def test_widgetbox_widget(fake_qtile, fake_window):
 def test_widgetbox_mirror(manager_nospawn, minimal_conf_noscreen):
     config = minimal_conf_noscreen
     tbox = TextBox(text="Text Box")
-    config.screens = [
-        libqtile.config.Screen(
-            top=libqtile.bar.Bar([tbox, WidgetBox([tbox])], 10)
-        )
-    ]
+    config.screens = [libqtile.config.Screen(top=libqtile.bar.Bar([tbox, WidgetBox([tbox])], 10))]
 
     manager_nospawn.start(config)
 
@@ -81,11 +67,7 @@ def test_widgetbox_mirror(manager_nospawn, minimal_conf_noscreen):
 def test_widgetbox_mouse_click(manager_nospawn, minimal_conf_noscreen):
     config = minimal_conf_noscreen
     tbox = TextBox(text="Text Box")
-    config.screens = [
-        libqtile.config.Screen(
-            top=libqtile.bar.Bar([WidgetBox([tbox])], 10)
-        )
-    ]
+    config.screens = [libqtile.config.Screen(top=libqtile.bar.Bar([WidgetBox([tbox])], 10))]
 
     manager_nospawn.start(config)
 
